@@ -1,29 +1,29 @@
 use anyhow::{bail, Context, Result};
 use clap::StructOpt;
 use console::style;
-use ignore::{gitignore::Gitignore, overrides::OverrideBuilder, WalkBuilder};
+use ignore::{overrides::OverrideBuilder, WalkBuilder};
 use log::{LevelFilter, *};
 use serde_json::json;
+use std::collections::HashSet;
 use std::fs;
 use std::io::{stderr, stdin, stdout, Read, Write};
 use std::path::Path;
-#[cfg(feature = "editorconfig")]
-use std::path::PathBuf;
 use std::sync::atomic::{AtomicI32, AtomicU32, Ordering};
 use std::sync::Arc;
 use std::time::Instant;
 use thiserror::Error;
 use threadpool::ThreadPool;
 
-#[cfg(feature = "editorconfig")]
-use stylua_lib::editorconfig;
 use stylua_lib::{format_code, Config, OutputVerification, Range};
 
-use crate::config::find_ignore_file_path;
-
 mod config;
+#[cfg(feature = "lsp")]
+mod lsp;
 mod opt;
 mod output_diff;
+mod stylua_ignore;
+
+use stylua_ignore::{is_explicitly_provided, path_is_stylua_ignored, should_respect_ignores};
 
 static EXIT_CODE: AtomicI32 = AtomicI32::new(0);
 static UNFORMATTED_FILE_COUNT: AtomicU32 = AtomicU32::new(0);
@@ -46,60 +46,49 @@ struct ErrorFileWrapper {
     error: anyhow::Error,
 }
 
-fn convert_parse_error_to_json(file: &str, err: &full_moon::Error) -> Option<serde_json::Value> {
-    Some(match err {
-        full_moon::Error::AstError(full_moon::ast::AstError::UnexpectedToken {
-            token,
-            additional,
-        }) => json!({
-            "type": "parse_error",
-            "message": format!("unexpected token `{}`{}", token, additional.as_ref().map(|x| format!(": {x}")).unwrap_or_default()),
-            "filename": file,
-            "location": {
-                "start": token.start_position().bytes(),
-                "start_line": token.start_position().line(),
-                "start_column": token.start_position().character(),
-                "end": token.end_position().bytes(),
-                "end_line": token.end_position().line(),
-                "end_column": token.end_position().character(),
-            },
-        }),
-        full_moon::Error::TokenizerError(error) => json!({
-            "type": "parse_error",
-            "message": match error.error() {
-                full_moon::tokenizer::TokenizerErrorType::UnclosedComment => {
-                    "unclosed comment".to_string()
-                }
-                full_moon::tokenizer::TokenizerErrorType::UnclosedString => {
-                    "unclosed string".to_string()
-                }
-                full_moon::tokenizer::TokenizerErrorType::UnexpectedShebang => {
-                    "unexpected shebang".to_string()
-                }
-                full_moon::tokenizer::TokenizerErrorType::UnexpectedToken(
-                    character,
-                ) => {
-                    format!("unexpected character {character}")
-                }
-                full_moon::tokenizer::TokenizerErrorType::InvalidSymbol(symbol) => {
-                    format!("invalid symbol {symbol}")
-                }
-            },
-            "filename": file,
-            "location": {
-                "start": error.position().bytes(),
-                "start_line": error.position().line(),
-                "start_column": error.position().character(),
-                "end": error.position().bytes(),
-                "end_line": error.position().line(),
-                "end_column": error.position().character(),
-            },
-        }),
-        _ => {
-            error!("{:#}", err);
-            return None;
-        }
-    })
+fn convert_parse_error_to_json(file: &str, errs: Vec<full_moon::Error>) -> serde_json::Value {
+    errs.iter()
+        .map(|err| {
+            let message = match err {
+                full_moon::Error::AstError(ast_error) => format!(
+                    "unexpected token `{}`: {}",
+                    ast_error.token(),
+                    ast_error.error_message()
+                ),
+                full_moon::Error::TokenizerError(error) => match error.error() {
+                    full_moon::tokenizer::TokenizerErrorType::UnclosedComment => {
+                        "unclosed comment".to_string()
+                    }
+                    full_moon::tokenizer::TokenizerErrorType::UnclosedString => {
+                        "unclosed string".to_string()
+                    }
+                    full_moon::tokenizer::TokenizerErrorType::InvalidNumber => {
+                        "invalid number".to_string()
+                    }
+                    full_moon::tokenizer::TokenizerErrorType::UnexpectedToken(character) => {
+                        format!("unexpected character {character}")
+                    }
+                    full_moon::tokenizer::TokenizerErrorType::InvalidSymbol(symbol) => {
+                        format!("invalid symbol {symbol}")
+                    }
+                },
+            };
+            let (start_position, end_position) = err.range();
+            json!({
+                "type": "parse_error",
+                "message": message,
+                "filename": file,
+                "location": {
+                    "start": start_position.bytes(),
+                    "start_line": start_position.line(),
+                    "start_column": start_position.character(),
+                    "end": end_position.bytes(),
+                    "end_line": end_position.line(),
+                    "end_column": end_position.character(),
+                },
+            })
+        })
+        .collect()
 }
 
 fn create_diff(
@@ -179,8 +168,10 @@ fn format_file(
             None => Ok(FormatResult::Complete),
         }
     } else {
-        fs::write(path, formatted_contents)
-            .with_context(|| format!("could not write to {}", path.display()))?;
+        if formatted_contents != contents {
+            fs::write(path, formatted_contents)
+                .with_context(|| format!("could not write to {}", path.display()))?;
+        }
         Ok(FormatResult::Complete)
     }
 }
@@ -216,24 +207,21 @@ fn format_string(
     }
 }
 
-fn get_ignore(
-    directory: &Path,
-    search_parent_directories: bool,
-) -> Result<Gitignore, ignore::Error> {
-    let file_path = find_ignore_file_path(directory.to_path_buf(), search_parent_directories);
-    if let Some(file_path) = file_path {
-        let (ignore, err) = Gitignore::new(file_path);
-        if let Some(err) = err {
-            Err(err)
-        } else {
-            Ok(ignore)
-        }
-    } else {
-        Ok(Gitignore::empty())
-    }
-}
-
 fn format(opt: opt::Opt) -> Result<i32> {
+    debug!("resolved options: {:#?}", opt);
+
+    if opt.lsp {
+        #[cfg(feature = "lsp")]
+        {
+            lsp::run(opt)?;
+            return Ok(0);
+        }
+        #[cfg(not(feature = "lsp"))]
+        {
+            bail!("attempted to run stylua in LSP mode, but this binary was not built with 'lsp' feature enabled")
+        }
+    }
+
     if opt.files.is_empty() {
         bail!("no files provided");
     }
@@ -245,18 +233,12 @@ fn format(opt: opt::Opt) -> Result<i32> {
             opt::OutputFormat::Unified | opt::OutputFormat::Summary
         )
     {
-        bail!("--output-format=unified and --output-format=standard can only be used when --check is enabled");
+        bail!("--output-format=unified and --output-format=summary can only be used when --check is enabled");
     }
 
     // Load the configuration
-    let loaded = config::load_config(&opt)?;
-    #[cfg(feature = "editorconfig")]
-    let is_default_config = loaded.is_none();
-    let config = loaded.unwrap_or_default();
-
-    // Handle any configuration overrides provided by options
-    let config = config::load_overrides(config, &opt);
-    debug!("config: {:#?}", config);
+    let opt_for_config_resolver = opt.clone();
+    let mut config_resolver = config::ConfigResolver::new(&opt_for_config_resolver)?;
 
     // Create range if provided
     let range = if opt.range_start.is_some() || opt.range_end.is_some() {
@@ -281,7 +263,7 @@ fn format(opt: opt::Opt) -> Result<i32> {
     }
 
     walker_builder
-        .standard_filters(false)
+        .standard_filters(true)
         .hidden(!opt.allow_hidden)
         .parents(true)
         .add_custom_ignore_filename(".styluaignore");
@@ -360,22 +342,18 @@ fn format(opt: opt::Opt) -> Result<i32> {
                         Some(ErrorFileWrapper { file, error }) => {
                             match error.downcast_ref::<stylua_lib::Error>() {
                                 Some(stylua_lib::Error::ParseError(err)) => {
-                                    if let Some(structured_err) =
-                                        convert_parse_error_to_json(file, err)
-                                    {
-                                        // Force write to stderr directly
-                                        // TODO: can we do this through error! instead?
-                                        let stderr = stderr();
-                                        let mut handle = stderr.lock();
-                                        match handle
-                                            .write_all(structured_err.to_string().as_bytes())
-                                        {
-                                            Ok(_) => (),
-                                            Err(err) => {
-                                                error!("could not output to stdout: {:#}", err)
-                                            }
-                                        };
-                                    }
+                                    let structured_err =
+                                        convert_parse_error_to_json(file, err.to_vec());
+                                    // Force write to stderr directly
+                                    // TODO: can we do this through error! instead?
+                                    let stderr = stderr();
+                                    let mut handle = stderr.lock();
+                                    match handle.write_all(structured_err.to_string().as_bytes()) {
+                                        Ok(_) => (),
+                                        Err(err) => {
+                                            error!("could not output to stdout: {:#}", err)
+                                        }
+                                    };
                                 }
                                 _ => error!("{:#}", err),
                             }
@@ -389,6 +367,7 @@ fn format(opt: opt::Opt) -> Result<i32> {
     });
 
     let walker = walker_builder.build();
+    let mut seen_files = HashSet::new();
 
     for result in walker {
         match result {
@@ -398,52 +377,35 @@ fn format(opt: opt::Opt) -> Result<i32> {
                     let opt = opt.clone();
 
                     let should_skip_format = match &opt.stdin_filepath {
-                        Some(filepath) => {
-                            let ignore = get_ignore(
-                                filepath.parent().expect("cannot get parent directory"),
-                                opt.search_parent_directories,
-                            )
-                            .context("failed to parse ignore file")?;
-
-                            matches!(ignore.matched(filepath, false), ignore::Match::Ignore(_))
+                        Some(path) => {
+                            opt.respect_ignores
+                                && path_is_stylua_ignored(
+                                    path,
+                                    opt.search_parent_directories,
+                                    None,
+                                )?
                         }
                         None => false,
                     };
 
-                    pool.execute(move || {
-                        #[cfg(not(feature = "editorconfig"))]
-                        let used_config = Ok(config);
-                        #[cfg(feature = "editorconfig")]
-                        let used_config = match is_default_config && !&opt.no_editorconfig {
-                            true => {
-                                let path = match &opt.stdin_filepath {
-                                    Some(filepath) => filepath.to_path_buf(),
-                                    None => PathBuf::from("*.lua"),
-                                };
-                                editorconfig::parse(config, &path)
-                                    .context("could not parse editorconfig")
-                            }
-                            false => Ok(config),
-                        };
+                    let config = config_resolver.load_configuration_for_stdin()?;
 
+                    pool.execute(move || {
                         let mut buf = String::new();
                         tx.send(
-                            used_config
-                                .and_then(|used_config| {
-                                    stdin()
-                                        .read_to_string(&mut buf)
-                                        .map_err(|err| err.into())
-                                        .and_then(|_| {
-                                            format_string(
-                                                buf,
-                                                used_config,
-                                                range,
-                                                &opt,
-                                                verify_output,
-                                                should_skip_format,
-                                            )
-                                            .context("could not format from stdin")
-                                        })
+                            stdin()
+                                .read_to_string(&mut buf)
+                                .map_err(|err| err.into())
+                                .and_then(|_| {
+                                    format_string(
+                                        buf,
+                                        config,
+                                        range,
+                                        &opt,
+                                        verify_output,
+                                        should_skip_format,
+                                    )
+                                    .context("could not format from stdin")
                                 })
                                 .map_err(|error| {
                                     ErrorFileWrapper {
@@ -458,10 +420,16 @@ fn format(opt: opt::Opt) -> Result<i32> {
                 } else {
                     let path = entry.path().to_owned(); // TODO: stop to_owned?
                     let opt = opt.clone();
+
+                    if seen_files.contains(&path) {
+                        continue;
+                    }
+                    seen_files.insert(path.clone());
+
                     if path.is_file() {
                         // If the user didn't provide a glob pattern, we should match against our default one
-                        // We should ignore the glob check if the path provided was explicitly given to the CLI
-                        if use_default_glob && !opt.files.iter().any(|p| path == *p) {
+                        if use_default_glob && should_respect_ignores(opt.as_ref(), path.as_path())
+                        {
                             lazy_static::lazy_static! {
                                 static ref DEFAULT_GLOB: globset::GlobSet = {
                                     let mut builder = globset::GlobSetBuilder::new();
@@ -476,29 +444,29 @@ fn format(opt: opt::Opt) -> Result<i32> {
                             }
                         }
 
+                        // If `--respect-ignores` was given and this is an explicit file path,
+                        // we should check .styluaignore
+                        if is_explicitly_provided(opt.as_ref(), &path)
+                            && should_respect_ignores(opt.as_ref(), &path)
+                            && path_is_stylua_ignored(&path, opt.search_parent_directories, None)?
+                        {
+                            continue;
+                        }
+
+                        let config = config_resolver.load_configuration(&path)?;
+
                         let tx = tx.clone();
                         pool.execute(move || {
-                            #[cfg(not(feature = "editorconfig"))]
-                            let used_config = Ok(config);
-                            #[cfg(feature = "editorconfig")]
-                            let used_config = match is_default_config && !&opt.no_editorconfig {
-                                true => editorconfig::parse(config, &path)
-                                    .context("could not parse editorconfig"),
-                                false => Ok(config),
-                            };
-
                             tx.send(
-                                used_config
-                                    .and_then(|used_config| {
-                                        format_file(&path, used_config, range, &opt, verify_output)
-                                    })
-                                    .map_err(|error| {
+                                format_file(&path, config, range, &opt, verify_output).map_err(
+                                    |error| {
                                         ErrorFileWrapper {
                                             file: path.display().to_string(),
                                             error,
                                         }
                                         .into()
-                                    }),
+                                    },
+                                ),
                             )
                             .unwrap()
                         });
@@ -624,10 +592,27 @@ fn main() {
 #[cfg(test)]
 mod tests {
     use assert_cmd::Command;
+    use assert_fs::prelude::*;
+
+    macro_rules! construct_tree {
+        ({ $($file_name:literal:$file_contents:literal,)* }) => {{
+            let cwd = assert_fs::TempDir::new().unwrap();
+
+            $(
+                cwd.child($file_name).write_str($file_contents).unwrap();
+            )*
+
+            cwd
+        }};
+    }
+
+    fn create_stylua() -> Command {
+        Command::cargo_bin(env!("CARGO_PKG_NAME")).unwrap()
+    }
 
     #[test]
     fn test_no_files_provided() {
-        let mut cmd = Command::cargo_bin(env!("CARGO_PKG_NAME")).unwrap();
+        let mut cmd = create_stylua();
         cmd.assert()
             .failure()
             .code(2)
@@ -636,11 +621,418 @@ mod tests {
 
     #[test]
     fn test_format_stdin() {
-        let mut cmd = Command::cargo_bin(env!("CARGO_PKG_NAME")).unwrap();
+        let mut cmd = create_stylua();
         cmd.arg("-")
             .write_stdin("local   x   = 1")
             .assert()
             .success()
             .stdout("local x = 1\n");
     }
+
+    #[test]
+    fn test_format_file() {
+        let cwd = construct_tree!({
+            "foo.lua": "local   x    =   1",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path()).arg(".").assert().success();
+
+        cwd.child("foo.lua").assert("local x = 1\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_stylua_ignore() {
+        let cwd = construct_tree!({
+            ".styluaignore": "ignored/",
+            "foo.lua": "local   x    =   1",
+            "ignored/bar.lua": "local   x    =   1",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path()).arg(".").assert().success();
+
+        cwd.child("foo.lua").assert("local x = 1\n");
+        cwd.child("ignored/bar.lua").assert("local   x    =   1");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn explicitly_provided_files_dont_check_ignores() {
+        let cwd = construct_tree!({
+            ".styluaignore": "foo.lua",
+            "foo.lua": "local   x    =   1",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("foo.lua").assert("local x = 1\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn explicitly_provided_files_dont_check_ignores_stdin() {
+        let cwd = construct_tree!({
+            ".styluaignore": "foo.lua",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--stdin-filepath", "foo.lua", "-"])
+            .write_stdin("local   x    =   1")
+            .assert()
+            .success()
+            .stdout("local x = 1\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn explicitly_provided_files_not_in_cwd() {
+        let cwd = construct_tree!({
+            ".styluaignore": "foo.lua",
+        });
+
+        let another = construct_tree!({});
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args([
+                "--respect-ignores",
+                "--stdin-filepath",
+                another.child("foo.lua").to_str().unwrap(),
+                "-",
+            ])
+            .write_stdin("local   x    =   1")
+            .assert()
+            .success()
+            .stdout("local x = 1\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_respect_ignores() {
+        let cwd = construct_tree!({
+            ".styluaignore": "foo.lua",
+            "foo.lua": "local   x    =   1",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--respect-ignores", "foo.lua"])
+            .assert()
+            .success();
+
+        cwd.child("foo.lua").assert("local   x    =   1");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_respect_ignores_stdin() {
+        let cwd = construct_tree!({
+            ".styluaignore": "foo.lua",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--respect-ignores", "--stdin-filepath", "foo.lua", "-"])
+            .write_stdin("local   x    =   1")
+            .assert()
+            .success()
+            .stdout("local   x    =   1");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_respect_ignores_directory_no_glob() {
+        // https://github.com/JohnnyMorganz/StyLua/issues/845
+        let cwd = construct_tree!({
+            ".styluaignore": "build/",
+            "build/foo.lua": "local   x    =   1",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--check", "--respect-ignores", "build/foo.lua"])
+            .assert()
+            .success();
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_stdin_filepath_respects_cwd_configuration_next_to_file() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--stdin-filepath", "foo.lua", "-"])
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_stdin_filepath_respects_cwd_configuration_for_nested_file() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--stdin-filepath", "build/foo.lua", "-"])
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_cwd_configuration_respected_when_formatting_from_stdin() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("-")
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_cwd_configuration_respected_for_file_in_cwd() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_cwd_configuration_respected_for_nested_file() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "build/foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("build/foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("build/foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_configuration_is_not_used_outside_of_cwd() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "build/foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.child("build").path())
+            .arg("foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("build/foo.lua").assert("local x = \"hello\"\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_configuration_used_outside_of_cwd_when_search_parent_directories_is_enabled() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "build/foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.child("build").path())
+            .args(["--search-parent-directories", "foo.lua"])
+            .assert()
+            .success();
+
+        cwd.child("build/foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_configuration_is_searched_next_to_file() {
+        let cwd = construct_tree!({
+            "build/stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "build/foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("build/foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("build/foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_configuration_is_used_closest_to_the_file() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferDouble'",
+            "build/stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "build/foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .arg("build/foo.lua")
+            .assert()
+            .success();
+
+        cwd.child("build/foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_respect_config_path_override() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferDouble'",
+            "build/stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--config-path", "build/stylua.toml", "foo.lua"])
+            .assert()
+            .success();
+    }
+
+    #[test]
+    fn test_respect_config_path_override_for_stdin_filepath() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferDouble'",
+            "build/stylua.toml": "quote_style = 'AutoPreferSingle'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--config-path", "build/stylua.toml", "-"])
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_uses_cli_overrides_instead_of_default_configuration() {
+        let cwd = construct_tree!({
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--quote-style", "AutoPreferSingle", "."])
+            .assert()
+            .success();
+
+        cwd.child("foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_uses_cli_overrides_instead_of_default_configuration_stdin_filepath() {
+        let cwd = construct_tree!({
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--quote-style", "AutoPreferSingle", "-"])
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_uses_cli_overrides_instead_of_found_configuration() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferDouble'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--quote-style", "AutoPreferSingle", "."])
+            .assert()
+            .success();
+
+        cwd.child("foo.lua").assert("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
+
+    #[test]
+    fn test_uses_cli_overrides_instead_of_found_configuration_stdin_filepath() {
+        let cwd = construct_tree!({
+            "stylua.toml": "quote_style = 'AutoPreferDouble'",
+            "foo.lua": "local x = \"hello\"",
+        });
+
+        let mut cmd = create_stylua();
+        cmd.current_dir(cwd.path())
+            .args(["--quote-style", "AutoPreferSingle", "-"])
+            .write_stdin("local x = \"hello\"")
+            .assert()
+            .success()
+            .stdout("local x = 'hello'\n");
+
+        cwd.close().unwrap();
+    }
 }
