@@ -1,11 +1,11 @@
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{
+use full_moon::ast::luau::{
     ElseIfExpression, IfExpression, InterpolatedString, InterpolatedStringSegment,
 };
 use full_moon::{
     ast::{
-        span::ContainedSpan, BinOp, Expression, FunctionCall, Index, Prefix, Suffix, UnOp, Value,
-        Var, VarExpression,
+        span::ContainedSpan, BinOp, Expression, FunctionCall, Index, Prefix, Suffix, UnOp, Var,
+        VarExpression,
     },
     node::Node,
     tokenizer::{StringLiteralQuoteType, Symbol, Token, TokenReference, TokenType},
@@ -31,7 +31,8 @@ use crate::{
             UpdateTrailingTrivia, UpdateTrivia,
         },
         trivia_util::{
-            self, contains_comments, trivia_is_newline, CommentSearch, GetLeadingTrivia,
+            self, contains_comments, prepend_newline_indent, take_leading_comments,
+            take_trailing_comments, trivia_is_newline, CommentSearch, GetLeadingTrivia,
             GetTrailingTrivia,
         },
     },
@@ -46,6 +47,7 @@ macro_rules! fmt_op {
                 $(#[$inner])*
                 $enum::$operator(token) => $enum::$operator(fmt_symbol!($ctx, token, $output, $shape)),
             )+
+            #[allow(clippy::redundant_closure_call)]
             other => $other(other),
         }
     };
@@ -99,7 +101,7 @@ pub fn format_binop(ctx: &Context, binop: &BinOp, shape: Shape) -> BinOp {
         TwoEqual = " == ",
         #[cfg(feature = "lua53")]
         Ampersand = " & ",
-        #[cfg(feature = "lua53")]
+        #[cfg(any(feature = "luau", feature = "lua53"))]
         DoubleSlash = " // ",
         #[cfg(feature = "lua53")]
         DoubleLessThan = " << ",
@@ -144,46 +146,38 @@ fn check_excess_parentheses(internal_expression: &Expression, context: Expressio
         }
         // Don't bother removing them if there is a binop, as they may be needed. TODO: can we be more intelligent here?
         Expression::BinaryOperator { .. } => false,
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
-            type_assertion,
-        } => {
-            // If we have a type assertion, and the context is a unary or binary operation
-            // we should always keep parentheses
-            // [e.g. #(value :: Array<string>) or -(value :: number)]
-            #[cfg(feature = "luau")]
-            if type_assertion.is_some()
-                && matches!(
-                    context,
-                    ExpressionContext::UnaryOrBinary
-                        | ExpressionContext::BinaryLHS
-                        | ExpressionContext::BinaryLHSExponent
-                )
-            {
-                return false;
-            }
 
-            match &**value {
-                // Internal expression is a function call
-                // We could potentially be culling values, so we should not remove parentheses
-                Value::FunctionCall(_) => false,
-                Value::Symbol(token_ref) => {
-                    match token_ref.token_type() {
-                        // If we have an ellipse inside of parentheses, we may also be culling values
-                        // Therefore, we don't remove parentheses
-                        TokenType::Symbol { symbol } => !matches!(symbol, Symbol::Ellipse),
-                        _ => true,
-                    }
-                }
-                // If the internal expression is an if expression, we need to keep the parentheses
-                // as modifying it can lead to issues [e.g. (if <x> then <expr> else <expr>) + 1 is different without parens]
-                #[cfg(feature = "luau")]
-                Value::IfExpression(_) => false,
+        // If we have a type assertion, and the context is a unary or binary operation
+        // we should always keep parentheses
+        // [e.g. #(value :: Array<string>) or -(value :: number)]
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion { .. }
+            if matches!(
+                context,
+                ExpressionContext::UnaryOrBinary
+                    | ExpressionContext::BinaryLHS
+                    | ExpressionContext::BinaryLHSExponent
+            ) =>
+        {
+            false
+        }
+
+        // Internal expression is a function call
+        // We could potentially be culling values, so we should not remove parentheses
+        Expression::FunctionCall(_) => false,
+        Expression::Symbol(token_ref) => {
+            match token_ref.token_type() {
+                // If we have an ellipsis inside of parentheses, we may also be culling values
+                // Therefore, we don't remove parentheses
+                TokenType::Symbol { symbol } => !matches!(symbol, Symbol::Ellipsis),
                 _ => true,
             }
         }
-        other => panic!("unknown node {:?}", other),
+        // If the internal expression is an if expression, we need to keep the parentheses
+        // as modifying it can lead to issues [e.g. (if <x> then <expr> else <expr>) + 1 is different without parens]
+        #[cfg(feature = "luau")]
+        Expression::IfExpression(_) => false,
+        _ => true,
     }
 }
 
@@ -200,29 +194,46 @@ fn format_expression_internal(
     shape: Shape,
 ) -> Expression {
     match expression {
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
-            type_assertion,
-        } => Expression::Value {
-            value: Box::new(format_value(
-                ctx,
-                value,
-                shape,
-                #[cfg(feature = "luau")]
-                if type_assertion.is_some() {
-                    ExpressionContext::TypeAssertion
-                } else {
-                    context
-                },
-                #[cfg(not(feature = "luau"))]
-                context,
-            )),
+        Expression::Function(anonymous_function) => {
+            Expression::Function(format_anonymous_function(ctx, anonymous_function, shape))
+        }
+        Expression::FunctionCall(function_call) => {
+            Expression::FunctionCall(format_function_call(ctx, function_call, shape))
+        }
+        #[cfg(feature = "luau")]
+        Expression::IfExpression(if_expression) => {
+            Expression::IfExpression(format_if_expression(ctx, if_expression, shape))
+        }
+        Expression::Number(token_reference) => {
+            Expression::Number(format_token_reference(ctx, token_reference, shape))
+        }
+        Expression::String(token_reference) => {
+            Expression::String(format_token_reference(ctx, token_reference, shape))
+        }
+        #[cfg(feature = "luau")]
+        Expression::InterpolatedString(interpolated_string) => Expression::InterpolatedString(
+            format_interpolated_string(ctx, interpolated_string, shape),
+        ),
+        Expression::Symbol(token_reference) => {
+            Expression::Symbol(format_token_reference(ctx, token_reference, shape))
+        }
+        Expression::TableConstructor(table_constructor) => {
+            Expression::TableConstructor(format_table_constructor(ctx, table_constructor, shape))
+        }
+        Expression::Var(var) => Expression::Var(format_var(ctx, var, shape)),
 
-            #[cfg(feature = "luau")]
-            type_assertion: type_assertion
-                .as_ref()
-                .map(|assertion| format_type_assertion(ctx, assertion, shape)),
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion {
+            expression,
+            type_assertion,
+        } => Expression::TypeAssertion {
+            expression: Box::new(format_expression_internal(
+                ctx,
+                expression,
+                ExpressionContext::TypeAssertion,
+                shape,
+            )),
+            type_assertion: format_type_assertion(ctx, type_assertion, shape),
         },
         Expression::Parentheses {
             contained,
@@ -242,29 +253,8 @@ fn format_expression_internal(
 
             // If the context is for a prefix, we should always keep the parentheses, as they are always required
             if use_internal_expression && !keep_parentheses {
-                // Get the leading and trailing comments from contained span and append them onto the expression
-                let (start_parens, end_parens) = contained.tokens();
-                let leading_comments = start_parens
-                    .leading_trivia()
-                    .filter(|token| trivia_util::trivia_is_comment(token))
-                    .flat_map(|x| {
-                        vec![
-                            create_indent_trivia(ctx, shape),
-                            x.to_owned(),
-                            create_newline_trivia(ctx),
-                        ]
-                    })
-                    // .chain(std::iter::once(create_indent_trivia(ctx, shape)))
-                    .collect();
-
-                let trailing_comments = end_parens
-                    .trailing_trivia()
-                    .filter(|token| trivia_util::trivia_is_comment(token))
-                    .flat_map(|x| {
-                        // Prepend a single space beforehand
-                        vec![Token::new(TokenType::spaces(1)), x.to_owned()]
-                    })
-                    .collect();
+                let (leading_comments, trailing_comments) =
+                    contained_span_comments(ctx, contained, shape);
 
                 format_expression(ctx, expression, shape)
                     .update_leading_trivia(FormatTriviaType::Append(leading_comments))
@@ -294,15 +284,13 @@ fn format_expression_internal(
                         unop: UnOp::Minus(_),
                         ..
                     } => true,
-
-                    Expression::Value { ref value, .. } => matches!(
-                        **value,
-                        Value::ParenthesesExpression(Expression::UnaryOperator {
+                    Expression::Parentheses { ref expression, .. } => matches!(
+                        &**expression,
+                        Expression::UnaryOperator {
                             unop: UnOp::Minus(_),
                             ..
-                        })
+                        }
                     ),
-
                     _ => false,
                 };
 
@@ -349,20 +337,78 @@ fn format_expression_internal(
     }
 }
 
+fn contained_span_comments(
+    ctx: &Context,
+    contained_span: &ContainedSpan,
+    shape: Shape,
+) -> (Vec<Token>, Vec<Token>) {
+    // Get the leading and trailing comments from contained span and append them onto the expression
+    let (start_parens, end_parens) = contained_span.tokens();
+    let leading_comments = start_parens
+        .leading_trivia()
+        .filter(|token| trivia_util::trivia_is_comment(token))
+        .flat_map(|x| {
+            vec![
+                create_indent_trivia(ctx, shape),
+                x.to_owned(),
+                create_newline_trivia(ctx),
+            ]
+        })
+        // .chain(std::iter::once(create_indent_trivia(ctx, shape)))
+        .collect();
+
+    let trailing_comments = end_parens
+        .trailing_trivia()
+        .filter(|token| trivia_util::trivia_is_comment(token))
+        .flat_map(|x| {
+            // Prepend a single space beforehand
+            vec![Token::new(TokenType::spaces(1)), x.to_owned()]
+        })
+        .collect();
+    (leading_comments, trailing_comments)
+}
+
 /// Determines whether the provided [`Expression`] is a brackets string, i.e. `[[string]]`
+/// We care about this because `[ [[string] ]` is invalid syntax if we remove the whitespace
 pub fn is_brackets_string(expression: &Expression) -> bool {
-    if let Expression::Value { value, .. } = expression {
-        if let Value::String(token_reference) = &**value {
-            return matches!(
-                token_reference.token_type(),
-                TokenType::StringLiteral {
-                    quote_type: StringLiteralQuoteType::Brackets,
-                    ..
-                }
-            );
-        }
+    match expression {
+        Expression::String(token_reference) => matches!(
+            token_reference.token_type(),
+            TokenType::StringLiteral {
+                quote_type: StringLiteralQuoteType::Brackets,
+                ..
+            }
+        ),
+        Expression::Parentheses { expression, .. } => is_brackets_string(expression),
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion { expression, .. } => is_brackets_string(expression),
+        _ => false,
     }
-    false
+}
+
+pub fn process_dot_name(
+    ctx: &Context,
+    dot: &TokenReference,
+    name: &TokenReference,
+    shape: Shape,
+) -> (TokenReference, TokenReference) {
+    // If there are any comments in between the dot and name,
+    // then taken them out and put them before the dot
+    let (mut dot, mut dot_comments) =
+        take_trailing_comments(&format_token_reference(ctx, dot, shape));
+    let (name, name_comments) = take_leading_comments(&format_token_reference(ctx, name, shape));
+
+    dot_comments.extend(name_comments);
+
+    if !dot_comments.is_empty() {
+        dot = prepend_newline_indent(
+            ctx,
+            &dot.update_leading_trivia(FormatTriviaType::Append(dot_comments)),
+            shape,
+        );
+    }
+
+    (dot, name)
 }
 
 /// Formats an Index Node
@@ -425,10 +471,10 @@ pub fn format_index(ctx: &Context, index: &Index, shape: Shape) -> Index {
             }
         }
 
-        Index::Dot { dot, name } => Index::Dot {
-            dot: format_token_reference(ctx, dot, shape),
-            name: format_token_reference(ctx, name, shape),
-        },
+        Index::Dot { dot, name } => {
+            let (dot, name) = process_dot_name(ctx, dot, name, shape);
+            Index::Dot { dot, name }
+        }
         other => panic!("unknown node {:?}", other),
     }
 }
@@ -436,12 +482,9 @@ pub fn format_index(ctx: &Context, index: &Index, shape: Shape) -> Index {
 // Checks if this is a string (allows strings wrapped in parentheses)
 fn is_string(expression: &Expression) -> bool {
     match expression {
+        Expression::String(_) => true,
         #[cfg(feature = "luau")]
-        Expression::Value { value, .. } => {
-            matches!(&**value, Value::String(_) | Value::InterpolatedString(_))
-        }
-        #[cfg(not(feature = "luau"))]
-        Expression::Value { value, .. } => matches!(&**value, Value::String(_)),
+        Expression::InterpolatedString(_) => true,
         Expression::Parentheses { expression, .. } => is_string(expression),
         _ => false,
     }
@@ -456,15 +499,15 @@ pub fn format_prefix(ctx: &Context, prefix: &Prefix, shape: Shape) -> Prefix {
             let singeline_shape = shape.take_first_line(&strip_trivia(&singleline_format));
 
             if singeline_shape.over_budget() && !is_string(expression) {
-                Prefix::Expression(format_hanging_expression_(
+                Prefix::Expression(Box::new(format_hanging_expression_(
                     ctx,
                     expression,
                     shape,
                     ExpressionContext::Prefix,
                     None,
-                ))
+                )))
             } else {
-                Prefix::Expression(singleline_format)
+                Prefix::Expression(Box::new(singleline_format))
             }
         }
         Prefix::Name(token_reference) => {
@@ -634,15 +677,13 @@ fn format_if_expression(ctx: &Context, if_expression: &IfExpression, shape: Shap
         || trivia_util::contains_comments(if_expression.else_token())
         || if_expression
             .else_if_expressions()
-            .map_or(false, |else_ifs| {
-                else_ifs.iter().any(trivia_util::contains_comments)
-            })
+            .is_some_and(|else_ifs| else_ifs.iter().any(trivia_util::contains_comments))
         || if_expression.else_expression().has_inline_comments()
         || trivia_util::spans_multiple_lines(&singleline_condition)
         || trivia_util::spans_multiple_lines(&singleline_expression)
-        || else_ifs.as_ref().map_or(false, |else_ifs| {
-            else_ifs.iter().any(trivia_util::spans_multiple_lines)
-        })
+        || else_ifs
+            .as_ref()
+            .is_some_and(|else_ifs| else_ifs.iter().any(trivia_util::spans_multiple_lines))
         || trivia_util::spans_multiple_lines(&singleline_else_expression);
 
     if require_multiline_expression {
@@ -800,8 +841,17 @@ fn format_interpolated_string(
         let literal = format_token_reference(ctx, &segment.literal, shape);
         shape = shape + literal.to_string().len();
 
-        let expression = format_expression(ctx, &segment.expression, shape);
-        shape.take_last_line(&expression);
+        let mut expression = format_expression(ctx, &segment.expression, shape);
+        shape = shape.take_last_line(&expression);
+
+        // If expression is a table constructor, then ensure a space is added beforehand
+        // since `{{` syntax is not permitted
+        if let Expression::TableConstructor { .. } = expression {
+            expression =
+                expression.update_leading_trivia(FormatTriviaType::Append(vec![Token::new(
+                    TokenType::spaces(1),
+                )]))
+        }
 
         segments.push(InterpolatedStringSegment {
             literal,
@@ -819,43 +869,6 @@ fn format_interpolated_string(
         ))
 }
 
-/// Formats a Value Node
-fn format_value(ctx: &Context, value: &Value, shape: Shape, context: ExpressionContext) -> Value {
-    match value {
-        Value::Function((token_reference, function_body)) => Value::Function(
-            format_anonymous_function(ctx, token_reference, function_body, shape),
-        ),
-        Value::FunctionCall(function_call) => {
-            Value::FunctionCall(format_function_call(ctx, function_call, shape))
-        }
-        #[cfg(feature = "luau")]
-        Value::IfExpression(if_expression) => {
-            Value::IfExpression(format_if_expression(ctx, if_expression, shape))
-        }
-        Value::Number(token_reference) => {
-            Value::Number(format_token_reference(ctx, token_reference, shape))
-        }
-        Value::ParenthesesExpression(expression) => Value::ParenthesesExpression(
-            format_expression_internal(ctx, expression, context, shape),
-        ),
-        Value::String(token_reference) => {
-            Value::String(format_token_reference(ctx, token_reference, shape))
-        }
-        #[cfg(feature = "luau")]
-        Value::InterpolatedString(interpolated_string) => {
-            Value::InterpolatedString(format_interpolated_string(ctx, interpolated_string, shape))
-        }
-        Value::Symbol(token_reference) => {
-            Value::Symbol(format_token_reference(ctx, token_reference, shape))
-        }
-        Value::TableConstructor(table_constructor) => {
-            Value::TableConstructor(format_table_constructor(ctx, table_constructor, shape))
-        }
-        Value::Var(var) => Value::Var(format_var(ctx, var, shape)),
-        other => panic!("unknown node {:?}", other),
-    }
-}
-
 /// Formats a Var Node
 pub fn format_var(ctx: &Context, var: &Var, shape: Shape) -> Var {
     match var {
@@ -863,7 +876,7 @@ pub fn format_var(ctx: &Context, var: &Var, shape: Shape) -> Var {
             Var::Name(format_token_reference(ctx, token_reference, shape))
         }
         Var::Expression(var_expression) => {
-            Var::Expression(format_var_expression(ctx, var_expression, shape))
+            Var::Expression(Box::new(format_var_expression(ctx, var_expression, shape)))
         }
         other => panic!("unknown node {:?}", other),
     }
@@ -1120,6 +1133,7 @@ fn hang_binop_expression(
     top_binop: BinOp,
     shape: Shape,
     lhs_range: Option<LeftmostRangeHang>,
+    expression_context: ExpressionContext,
 ) -> Expression {
     const SPACE_LEN: usize = " ".len();
 
@@ -1178,9 +1192,17 @@ fn hang_binop_expression(
                                 },
                                 lhs_shape,
                                 lhs_range,
+                                expression_context,
                             ),
                             if contains_comments(&*rhs) {
-                                hang_binop_expression(ctx, *rhs, binop, shape, lhs_range)
+                                hang_binop_expression(
+                                    ctx,
+                                    *rhs,
+                                    binop,
+                                    shape,
+                                    lhs_range,
+                                    expression_context,
+                                )
                             } else {
                                 format_expression_internal(
                                     ctx,
@@ -1192,7 +1214,14 @@ fn hang_binop_expression(
                         ),
                         ExpressionSide::Right => (
                             if contains_comments(&*lhs) {
-                                hang_binop_expression(ctx, *lhs, binop.clone(), shape, lhs_range)
+                                hang_binop_expression(
+                                    ctx,
+                                    *lhs,
+                                    binop.clone(),
+                                    shape,
+                                    lhs_range,
+                                    expression_context,
+                                )
                             } else {
                                 let context = if let BinOp::Caret(_) = binop {
                                     ExpressionContext::BinaryLHSExponent
@@ -1207,6 +1236,7 @@ fn hang_binop_expression(
                                 if same_op_level { top_binop } else { binop },
                                 rhs_shape,
                                 lhs_range,
+                                expression_context,
                             ),
                         ),
                     };
@@ -1219,7 +1249,14 @@ fn hang_binop_expression(
                     // Check if the chain still has comments deeper inside of it.
                     // If it does, we need to hang that part of the chain still, otherwise the comments will mess it up
                     let lhs = if contains_comments(&*lhs) {
-                        hang_binop_expression(ctx, *lhs, binop.to_owned(), shape, lhs_range)
+                        hang_binop_expression(
+                            ctx,
+                            *lhs,
+                            binop.to_owned(),
+                            shape,
+                            lhs_range,
+                            expression_context,
+                        )
                     } else {
                         let context = if let BinOp::Caret(_) = binop {
                             ExpressionContext::BinaryLHSExponent
@@ -1230,7 +1267,14 @@ fn hang_binop_expression(
                     };
 
                     let rhs = if contains_comments(&*rhs) {
-                        hang_binop_expression(ctx, *rhs, binop, shape, lhs_range)
+                        hang_binop_expression(
+                            ctx,
+                            *rhs,
+                            binop,
+                            shape,
+                            lhs_range,
+                            expression_context,
+                        )
                     } else {
                         format_expression_internal(
                             ctx,
@@ -1251,13 +1295,7 @@ fn hang_binop_expression(
             }
         }
         // Base case: no more binary operators - just return to normal splitting
-        _ => format_hanging_expression_(
-            ctx,
-            &expression,
-            shape,
-            ExpressionContext::Standard,
-            lhs_range,
-        ),
+        _ => format_hanging_expression_(ctx, &expression, shape, expression_context, lhs_range),
     }
 }
 
@@ -1272,55 +1310,33 @@ fn format_hanging_expression_(
     let expression_range = expression.to_range();
 
     match expression {
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion {
+            expression,
             type_assertion,
         } => {
-            #[cfg(feature = "luau")]
-            let (expression_context, value_shape) = if let Some(type_assertion) = type_assertion {
-                // If we have a type assertion, we increment the current shape with the size of the assertion
-                // to "force" the parentheses to hang if necessary
-                (
-                    ExpressionContext::TypeAssertion,
-                    shape.take_first_line(&strip_trivia(type_assertion)),
-                )
-            } else {
-                (expression_context, shape)
-            };
-            #[cfg(not(feature = "luau"))]
-            let value_shape = shape;
+            // If we have a type assertion, we increment the current shape with the size of the assertion
+            // to "force" the parentheses to hang if necessary
+            let (expression_context, value_shape) = (
+                ExpressionContext::TypeAssertion,
+                shape.take_first_line(&strip_trivia(type_assertion)),
+            );
 
-            let value = Box::new(match &**value {
-                Value::ParenthesesExpression(expression) => {
-                    Value::ParenthesesExpression(format_hanging_expression_(
-                        ctx,
-                        expression,
-                        value_shape,
-                        expression_context,
-                        lhs_range,
-                    ))
-                }
-                _ => {
-                    let value_shape = if let Some(lhs_hang) = lhs_range {
-                        lhs_hang.required_shape(value_shape, &expression_range)
-                    } else {
-                        value_shape
-                    };
-                    format_value(ctx, value, value_shape, expression_context)
-                }
-            });
+            let expression = format_hanging_expression_(
+                ctx,
+                expression,
+                value_shape,
+                expression_context,
+                lhs_range,
+            );
 
             // Update the shape used to format the type assertion
             #[cfg(feature = "luau")]
-            let assertion_shape = shape.take_last_line(&value);
+            let assertion_shape = shape.take_last_line(&expression);
 
-            Expression::Value {
-                value,
-                #[cfg(feature = "luau")]
-                type_assertion: type_assertion
-                    .as_ref()
-                    .map(|assertion| format_type_assertion(ctx, assertion, assertion_shape)),
+            Expression::TypeAssertion {
+                expression: Box::new(expression),
+                type_assertion: format_type_assertion(ctx, type_assertion, assertion_shape),
             }
         }
         Expression::Parentheses {
@@ -1346,6 +1362,8 @@ fn format_hanging_expression_(
 
             // If the context is for a prefix, we should always keep the parentheses, as they are always required
             if use_internal_expression && !keep_parentheses {
+                let (leading_comments, trailing_comments) =
+                    contained_span_comments(ctx, contained, shape);
                 format_hanging_expression_(
                     ctx,
                     expression,
@@ -1353,6 +1371,8 @@ fn format_hanging_expression_(
                     expression_context,
                     lhs_range,
                 )
+                .update_leading_trivia(FormatTriviaType::Append(leading_comments))
+                .update_trailing_trivia(FormatTriviaType::Append(trailing_comments))
             } else {
                 let contained = format_contained_span(ctx, contained, lhs_shape);
 
@@ -1420,8 +1440,14 @@ fn format_hanging_expression_(
         }
         Expression::BinaryOperator { lhs, binop, rhs } => {
             // Don't format the lhs and rhs here, because it will be handled later when hang_binop_expression calls back for a Value
-            let lhs =
-                hang_binop_expression(ctx, *lhs.to_owned(), binop.to_owned(), shape, lhs_range);
+            let lhs = hang_binop_expression(
+                ctx,
+                *lhs.to_owned(),
+                binop.to_owned(),
+                shape,
+                lhs_range,
+                ExpressionContext::UnaryOrBinary,
+            );
 
             let current_shape = shape.take_last_line(&lhs) + 1; // 1 = space before binop
             let mut new_binop = format_binop(ctx, binop, current_shape);
@@ -1434,6 +1460,7 @@ fn format_hanging_expression_(
                 binop.to_owned(),
                 singleline_shape,
                 None,
+                ExpressionContext::Standard,
             );
 
             // Examine the last line to see if we need to hang this binop, or if the precedence levels match
@@ -1452,6 +1479,7 @@ fn format_hanging_expression_(
                     binop.to_owned(),
                     hanging_shape,
                     None,
+                    ExpressionContext::Standard,
                 )
                 .update_leading_trivia(FormatTriviaType::Replace(Vec::new()));
             }
@@ -1462,7 +1490,15 @@ fn format_hanging_expression_(
                 rhs: Box::new(new_rhs),
             }
         }
-        other => panic!("unknown node {:?}", other),
+        _ => {
+            let value_shape = if let Some(lhs_hang) = lhs_range {
+                lhs_hang.required_shape(shape, &expression_range)
+            } else {
+                shape
+            };
+
+            format_expression_internal(ctx, expression, expression_context, value_shape)
+        }
     }
 }
 
