@@ -1,17 +1,21 @@
 use full_moon::ast::{
     punctuated::{Pair, Punctuated},
     span::ContainedSpan,
-    Block, Call, Expression, Field, FunctionArgs, FunctionBody, FunctionCall, FunctionDeclaration,
-    FunctionName, Index, LastStmt, LocalFunction, MethodCall, Parameter, Prefix, Stmt, Suffix,
-    TableConstructor, Value, Var,
+    AnonymousFunction, Block, Call, Expression, Field, FunctionArgs, FunctionBody, FunctionCall,
+    FunctionDeclaration, FunctionName, Index, LastStmt, LocalFunction, MethodCall, Parameter,
+    Prefix, Stmt, Suffix, TableConstructor, Var,
 };
 use full_moon::tokenizer::{Token, TokenKind, TokenReference, TokenType};
-use std::boxed::Box;
 
 #[cfg(feature = "luau")]
-use crate::formatters::luau::{format_generic_declaration, format_type_specifier};
+use crate::formatters::luau::{
+    format_generic_declaration, format_luau_attribute, format_type_specifier,
+};
 use crate::{
-    context::{create_indent_trivia, create_newline_trivia, Context},
+    context::{
+        create_function_call_trivia, create_function_definition_trivia, create_indent_trivia,
+        create_newline_trivia, Context,
+    },
     fmt_symbol,
     formatters::{
         block::{format_block, format_last_stmt_no_trivia},
@@ -31,21 +35,49 @@ use crate::{
         },
     },
     shape::Shape,
+    CallParenType,
 };
 
+use super::expression::process_dot_name;
+
 /// Formats an Anonymous Function
 /// This doesn't have its own struct, but it is part of Value::Function
 pub fn format_anonymous_function(
     ctx: &Context,
-    function_token: &TokenReference,
-    function_body: &FunctionBody,
+    anonymous_function: &AnonymousFunction,
     shape: Shape,
-) -> (TokenReference, FunctionBody) {
+) -> Box<AnonymousFunction> {
     const FUNCTION_LEN: usize = "function".len();
-    let function_token = fmt_symbol!(ctx, function_token, "function", shape);
-    let function_body = format_function_body(ctx, function_body, shape.add_width(FUNCTION_LEN));
 
-    (function_token, function_body)
+    // Format attributes on one line, space separated
+    #[cfg(feature = "luau")]
+    let attributes = anonymous_function
+        .attributes()
+        .map(|attribute| {
+            format_luau_attribute(ctx, attribute, shape).update_trailing_trivia(
+                FormatTriviaType::Append(vec![Token::new(TokenType::spaces(1))]),
+            )
+        })
+        .collect();
+
+    let function_definition_trivia = vec![create_function_definition_trivia(ctx)];
+    let function_token = fmt_symbol!(ctx, anonymous_function.function_token(), "function", shape)
+        .update_trailing_trivia(FormatTriviaType::Append(function_definition_trivia));
+    let function_body = format_function_body(
+        ctx,
+        anonymous_function.body(),
+        shape.add_width(FUNCTION_LEN),
+    );
+
+    let anonymous_function = anonymous_function
+        .clone()
+        .with_function_token(function_token)
+        .with_body(function_body);
+
+    #[cfg(feature = "luau")]
+    let anonymous_function = anonymous_function.with_attributes(attributes);
+
+    Box::new(anonymous_function)
 }
 
 /// An enum providing information regarding the next AST node after a function call.
@@ -73,13 +105,14 @@ pub fn format_call(
     shape: Shape,
     call_next_node: FunctionCallNextNode,
 ) -> Call {
+    let function_call_trivia = vec![create_function_call_trivia(ctx)];
     match call {
-        Call::AnonymousCall(function_args) => Call::AnonymousCall(format_function_args(
-            ctx,
-            function_args,
-            shape,
-            call_next_node,
-        )),
+        Call::AnonymousCall(function_args) => {
+            let formatted_function_args =
+                format_function_args(ctx, function_args, shape, call_next_node)
+                    .update_leading_trivia(FormatTriviaType::Append(function_call_trivia));
+            Call::AnonymousCall(formatted_function_args)
+        }
         Call::MethodCall(method_call) => {
             Call::MethodCall(format_method_call(ctx, method_call, shape, call_next_node))
         }
@@ -88,13 +121,10 @@ pub fn format_call(
 }
 
 fn is_table_constructor(expression: &Expression) -> bool {
-    match expression {
-        Expression::Value { value, .. } => matches!(**value, Value::TableConstructor(_)),
-        _ => false,
-    }
+    matches!(expression, Expression::TableConstructor(_))
 }
 
-fn is_complex_arg(value: &Value) -> bool {
+fn is_complex_arg(value: &Expression) -> bool {
     value.to_string().trim().contains('\n')
 }
 
@@ -125,7 +155,7 @@ fn function_args_contains_comments(
             // Punctuation contains comments
             || argument
                 .punctuation()
-                .map_or(false, |token| token.leading_trivia().chain(token.trailing_trivia()).any(function_trivia_contains_comments))
+                .is_some_and(|token| token.leading_trivia().chain(token.trailing_trivia()).any(function_trivia_contains_comments))
         })
     }
 }
@@ -223,82 +253,81 @@ fn function_args_multiline_heuristic(
     for pair in first_iter_formatted_arguments {
         let argument = pair.value();
         match argument {
-            Expression::Value { ref value, .. } => {
-                match &**value {
-                    // Check to see if we have a table constructor, or anonymous function
-                    Value::Function((_, function_body)) => {
-                        // Check to see whether it has been expanded
-                        let is_expanded = !should_collapse_function_body(ctx, function_body);
-                        if is_expanded {
-                            // If we have a mixture of multiline args, and other arguments
-                            // Then the function args should be expanded
-                            if current_state.should_hang() {
-                                return true;
-                            }
-
-                            current_state = current_state.record_multiline_arg();
-
-                            // First check the top line of the anonymous function (i.e. the function token and any parameters)
-                            // If this is over budget, then we should expand
-                            singleline_shape = singleline_shape.take_first_line(&value);
-                            if singleline_shape.over_budget() {
-                                return true;
-                            }
-
-                            // Reset the shape onto a new line, and include the `end` token
-                            singleline_shape = singleline_shape.reset() + END_LEN;
-                        } else {
-                            // Update the width with the collapsed function (normally indicative of a noop function)
-                            singleline_shape = singleline_shape + argument.to_string().len();
-                        }
-                    }
-                    Value::TableConstructor(table) => {
-                        // Check to see whether it has been expanded (there is a newline after the start brace)
-                        let is_expanded = trivia_util::trivia_contains_newline(
-                            table.braces().tokens().0.trailing_trivia(),
-                        );
-
-                        if is_expanded {
-                            // If we have a mixture of multiline args, and other arguments
-                            // Then the function args should be expanded
-                            if current_state.should_hang() {
-                                return true;
-                            }
-
-                            current_state = current_state.record_multiline_arg();
-
-                            // Reset the shape onto a new line
-                            singleline_shape = singleline_shape.reset() + BRACKET_LEN;
-                        } else {
-                            // Update the shape with the size of the collapsed table constructor
-                            singleline_shape = singleline_shape + argument.to_string().len();
-                        }
+            Expression::Function(anonymous_function) => {
+                // Check to see whether it has been expanded
+                let is_expanded = !should_collapse_function_body(ctx, anonymous_function.body());
+                if is_expanded {
+                    // If we have a mixture of multiline args, and other arguments
+                    // Then the function args should be expanded
+                    if current_state.should_hang() {
+                        return true;
                     }
 
-                    _ => {
-                        current_state = current_state.record_standard_arg();
+                    current_state = current_state.record_multiline_arg();
 
-                        // If the argument is complex (spans multiple lines), then we will immediately
-                        // exit and span multiline - it is most likely too complex to keep going forward.
-                        if is_complex_arg(value) && arguments.len() > 1 {
-                            return true;
-                        }
-
-                        // Take the first line to see if we are over budget
-                        if singleline_shape.take_first_line(&argument).over_budget() {
-                            return true;
-                        }
-
-                        // Update the shape with the last line (which may be different from the first)
-                        singleline_shape = singleline_shape.take_last_line(&argument);
+                    // First check the top line of the anonymous function (i.e. the function token and any parameters)
+                    // If this is over budget, then we should expand
+                    singleline_shape = singleline_shape.take_first_line(&argument);
+                    if singleline_shape.over_budget() {
+                        return true;
                     }
+
+                    // Reset the shape onto a new line, and include the `end` token
+                    singleline_shape = singleline_shape.reset() + END_LEN;
+                } else {
+                    // Update the width with the collapsed function (normally indicative of a noop function)
+                    singleline_shape = singleline_shape + argument.to_string().len();
+                }
+            }
+            Expression::TableConstructor(table) => {
+                // Check to see whether it has been expanded (there is a newline after the start brace)
+                let is_expanded = trivia_util::trivia_contains_newline(
+                    table.braces().tokens().0.trailing_trivia(),
+                );
+
+                if is_expanded {
+                    // If we have a mixture of multiline args, and other arguments
+                    // Then the function args should be expanded
+                    if current_state.should_hang() {
+                        return true;
+                    }
+
+                    // Include the first brace to check if we are over the column width already
+                    singleline_shape = singleline_shape.take_first_line(table);
+                    if singleline_shape.over_budget() {
+                        return true;
+                    }
+
+                    current_state = current_state.record_multiline_arg();
+
+                    // Reset the shape onto a new line
+                    singleline_shape = singleline_shape.reset() + BRACKET_LEN;
+                } else {
+                    // Update the shape with the size of the collapsed table constructor
+                    singleline_shape = singleline_shape + argument.to_string().len();
                 }
             }
             // TODO: Parentheses/UnOp, do we need to do more checking?
             // We will continue counting on the width_passed
-            _ => {
+            expression => {
                 current_state = current_state.record_standard_arg();
 
+                // TODO: BACKWARDS COMPATABILITY - THIS CODEPATH WAS ONLY FOLLOWED THROUGH WHEN
+                // IT WASN'T PARENTHESES / UNOP, AND THE PATHS MERGED WHEN REMOVING EXPRESSION::VALUE
+                // WE SHOULD SEE IF THIS IS ACTUALLY NECESSARY
+                if !matches!(
+                    expression,
+                    Expression::Parentheses { .. }
+                        | Expression::UnaryOperator { .. }
+                        | Expression::BinaryOperator { .. }
+                ) {
+                    // If the argument is complex (spans multiple lines), then we will immediately
+                    // exit and span multiline - it is most likely too complex to keep going forward.
+                    if is_complex_arg(expression) && arguments.len() > 1 {
+                        return true;
+                    }
+                }
+
                 // Take the first line to see if we are over budget
                 if singleline_shape.take_first_line(&argument).over_budget() {
                     return true;
@@ -368,46 +397,54 @@ pub fn format_function_args(
             parentheses,
             arguments,
         } => {
+            let (start_parens, end_parens) = parentheses.tokens();
+
             // Handle config where parentheses are omitted, and there is only one argument
-            if (ctx.should_omit_string_parens() || ctx.should_omit_table_parens())
+            if ctx.config().call_parentheses != CallParenType::Input
+                && (ctx.should_omit_string_parens() || ctx.should_omit_table_parens())
                 && arguments.len() == 1
                 && !matches!(call_next_node, FunctionCallNextNode::ObscureWithoutParens)
+                && !start_parens.has_trailing_comments(CommentSearch::Single)
+                && !end_parens.has_leading_comments(CommentSearch::Single)
+                && !arguments
+                    .first()
+                    .unwrap()
+                    .value()
+                    .has_leading_comments(CommentSearch::Single)
             {
                 let argument = arguments.iter().next().unwrap();
 
                 // Take any trailing trivia from the end parentheses, in case we need to keep it
                 let trailing_comments = parentheses.tokens().1.trailing_trivia().cloned().collect();
 
-                if let Expression::Value { value, .. } = argument {
-                    match &**value {
-                        Value::String(token_reference) => {
-                            if ctx.should_omit_string_parens() {
-                                return format_function_args(
-                                    ctx,
-                                    &FunctionArgs::String(token_reference.update_trailing_trivia(
-                                        FormatTriviaType::Append(trailing_comments),
-                                    )),
-                                    shape,
-                                    call_next_node,
-                                );
-                            }
+                match argument {
+                    Expression::String(token_reference) => {
+                        if ctx.should_omit_string_parens() {
+                            return format_function_args(
+                                ctx,
+                                &FunctionArgs::String(token_reference.update_trailing_trivia(
+                                    FormatTriviaType::Append(trailing_comments),
+                                )),
+                                shape,
+                                call_next_node,
+                            );
                         }
-                        Value::TableConstructor(table_constructor) => {
-                            if ctx.should_omit_table_parens() {
-                                return format_function_args(
-                                    ctx,
-                                    &FunctionArgs::TableConstructor(
-                                        table_constructor.update_trailing_trivia(
-                                            FormatTriviaType::Append(trailing_comments),
-                                        ),
-                                    ),
-                                    shape,
-                                    call_next_node,
-                                );
-                            }
-                        }
-                        _ => (),
                     }
+                    Expression::TableConstructor(table_constructor) => {
+                        if ctx.should_omit_table_parens() {
+                            return format_function_args(
+                                ctx,
+                                &FunctionArgs::TableConstructor(
+                                    table_constructor.update_trailing_trivia(
+                                        FormatTriviaType::Append(trailing_comments),
+                                    ),
+                                ),
+                                shape,
+                                call_next_node,
+                            );
+                        }
+                    }
+                    _ => (),
                 }
             }
 
@@ -445,7 +482,6 @@ pub fn format_function_args(
                 // parentheses as well. Otherwise, we just use 1 = opening parentheses.
                 let shape_increment = if hug_table_constructor { 2 } else { 1 };
 
-                let (start_parens, end_parens) = parentheses.tokens();
                 let start_parens = format_token_reference(ctx, start_parens, shape);
                 let start_parens = if start_parens.has_trailing_comments(CommentSearch::All)
                     && !arguments.is_empty()
@@ -488,8 +524,9 @@ pub fn format_function_args(
         }
 
         FunctionArgs::String(token_reference) => {
-            if ctx.should_omit_string_parens()
-                && !matches!(call_next_node, FunctionCallNextNode::ObscureWithoutParens)
+            if ctx.config().call_parentheses == CallParenType::Input
+                || (ctx.should_omit_string_parens()
+                    && !matches!(call_next_node, FunctionCallNextNode::ObscureWithoutParens))
             {
                 let token_reference = format_token_reference(ctx, token_reference, shape)
                     .update_leading_trivia(FormatTriviaType::Append(vec![Token::new(
@@ -502,11 +539,7 @@ pub fn format_function_args(
             let mut arguments = Punctuated::new();
             let new_expression = format_expression(
                 ctx,
-                &Expression::Value {
-                    value: Box::new(Value::String(token_reference.to_owned())),
-                    #[cfg(feature = "luau")]
-                    type_assertion: None,
-                },
+                &Expression::String(token_reference.to_owned()),
                 shape + 1, // 1 = opening parentheses
             );
 
@@ -530,8 +563,9 @@ pub fn format_function_args(
         }
 
         FunctionArgs::TableConstructor(table_constructor) => {
-            if ctx.should_omit_table_parens()
-                && !matches!(call_next_node, FunctionCallNextNode::ObscureWithoutParens)
+            if ctx.config().call_parentheses == CallParenType::Input
+                || (ctx.should_omit_table_parens()
+                    && !matches!(call_next_node, FunctionCallNextNode::ObscureWithoutParens))
             {
                 let table_constructor = format_table_constructor(ctx, table_constructor, shape)
                     .update_leading_trivia(FormatTriviaType::Append(vec![Token::new(
@@ -544,11 +578,7 @@ pub fn format_function_args(
             let mut arguments = Punctuated::new();
             let new_expression = format_expression(
                 ctx,
-                &Expression::Value {
-                    value: Box::new(Value::TableConstructor(table_constructor.to_owned())),
-                    #[cfg(feature = "luau")]
-                    type_assertion: None,
-                },
+                &Expression::TableConstructor(table_constructor.to_owned()),
                 shape + 1, // 1 = opening parentheses
             );
 
@@ -583,6 +613,10 @@ fn should_parameters_format_multiline(
     const PARENS_LEN: usize = "()".len();
     const SINGLELINE_END_LEN: usize = " end".len();
 
+    if function_body.parameters().is_empty() {
+        return false;
+    }
+
     // Check the length of the parameters. We need to format them first onto a single line to check if required
     let mut line_length = format_singleline_parameters(ctx, function_body, shape)
         .to_string()
@@ -639,6 +673,8 @@ fn table_constructor_contains_nested_function(table_constructor: &TableConstruct
         Field::ExpressionKey { key, value, .. } => {
             contains_nested_function(key) || contains_nested_function(value)
         }
+        #[cfg(feature = "cfxlua")]
+        Field::SetConstructor { .. } => false,
         Field::NameKey { value, .. } => contains_nested_function(value),
         other => unreachable!("unknown node {:?}", other),
     })
@@ -666,23 +702,20 @@ fn suffix_contains_nested_function(suffix: &Suffix) -> bool {
 /// Checks whether an expression contains a function body - in this case, we shouldn't collapse
 fn contains_nested_function(expression: &Expression) -> bool {
     match expression {
-        Expression::Value { value, .. } => match &**value {
-            Value::Function(_) => true,
-            Value::FunctionCall(call) => function_call_contains_nested_function(call),
-            Value::ParenthesesExpression(expression) => contains_nested_function(expression),
-            Value::TableConstructor(table_constructor) => {
-                table_constructor_contains_nested_function(table_constructor)
-            }
-            Value::Var(var) => var_contains_nested_function(var),
-            _ => false,
-        },
+        Expression::Function(_) => true,
+        Expression::FunctionCall(call) => function_call_contains_nested_function(call),
+        Expression::TableConstructor(table_constructor) => {
+            table_constructor_contains_nested_function(table_constructor)
+        }
+        Expression::Var(var) => var_contains_nested_function(var),
         Expression::BinaryOperator { lhs, rhs, .. } => {
             contains_nested_function(lhs) || contains_nested_function(rhs)
         }
         Expression::Parentheses { expression, .. } => contains_nested_function(expression),
         Expression::UnaryOperator { expression, .. } => contains_nested_function(expression),
-
-        other => unreachable!("unknown node {:?}", other),
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion { expression, .. } => contains_nested_function(expression),
+        _ => false,
     }
 }
 
@@ -766,15 +799,13 @@ pub fn format_function_body(
         let contains_comments = function_body.parameters().pairs().any(|pair| {
             let contains_comments = pair
                 .punctuation()
-                .map_or(false, trivia_util::token_contains_comments)
+                .is_some_and(trivia_util::token_contains_comments)
                 || trivia_util::contains_comments(pair.value());
             #[cfg(feature = "luau")]
             let type_specifier_comments = type_specifiers
                 .next()
                 .flatten()
-                .map_or(false, |type_specifier| {
-                    trivia_util::contains_comments(type_specifier)
-                });
+                .is_some_and(trivia_util::contains_comments);
             #[cfg(not(feature = "luau"))]
             let type_specifier_comments = false;
             contains_comments || type_specifier_comments
@@ -942,6 +973,19 @@ fn should_inline_prefix(ctx: &Context, prefix: &Prefix) -> bool {
         || prefix.len() <= ctx.config().indent_width
 }
 
+fn suffix_contains_comments(suffix: &Suffix) -> bool {
+    match suffix {
+        Suffix::Index(Index::Dot { dot, name }) => {
+            dot.has_trailing_comments(CommentSearch::All)
+                || name.has_leading_comments(CommentSearch::All)
+        }
+        Suffix::Call(Call::MethodCall(method_call)) => method_call
+            .colon_token()
+            .has_trailing_comments(CommentSearch::Single),
+        _ => false,
+    }
+}
+
 /// Formats a FunctionCall node
 pub fn format_function_call(
     ctx: &Context,
@@ -961,6 +1005,8 @@ pub fn format_function_call(
             let mut must_hang = false;
             while let Some(suffix) = peekable_suffixes.next() {
                 must_hang = suffix.has_leading_comments(CommentSearch::All)
+                // Check for comment placed inside of suffix
+                || suffix_contains_comments(suffix)
                 // Check for a trailing comment (iff there is still a suffix after this)
                 || (peekable_suffixes.peek().is_some()
                     && suffix.has_trailing_comments(CommentSearch::All));
@@ -1158,6 +1204,17 @@ pub fn format_function_declaration(
     // Calculate trivia
     let leading_trivia = vec![create_indent_trivia(ctx, shape)];
     let trailing_trivia = vec![create_newline_trivia(ctx)];
+    let function_definition_trivia = vec![create_function_definition_trivia(ctx)];
+
+    #[cfg(feature = "luau")]
+    let attributes = function_declaration
+        .attributes()
+        .map(|attribute| {
+            format_luau_attribute(ctx, attribute, shape)
+                .update_leading_trivia(FormatTriviaType::Append(leading_trivia.clone()))
+                .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia.clone()))
+        })
+        .collect();
 
     let function_token = fmt_symbol!(
         ctx,
@@ -1166,15 +1223,21 @@ pub fn format_function_declaration(
         shape
     )
     .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
-    let formatted_function_name = format_function_name(ctx, function_declaration.name(), shape);
+    let formatted_function_name = format_function_name(ctx, function_declaration.name(), shape)
+        .update_trailing_trivia(FormatTriviaType::Append(function_definition_trivia));
 
     let shape = shape + (9 + strip_trivia(&formatted_function_name).to_string().len()); // 9 = "function "
     let function_body = format_function_body(ctx, function_declaration.body(), shape)
         .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia));
 
-    FunctionDeclaration::new(formatted_function_name)
+    let function_declaration = FunctionDeclaration::new(formatted_function_name)
         .with_function_token(function_token)
-        .with_body(function_body)
+        .with_body(function_body);
+
+    #[cfg(feature = "luau")]
+    let function_declaration = function_declaration.with_attributes(attributes);
+
+    function_declaration
 }
 
 /// Formats a LocalFunction node
@@ -1186,20 +1249,36 @@ pub fn format_local_function(
     // Calculate trivia
     let leading_trivia = vec![create_indent_trivia(ctx, shape)];
     let trailing_trivia = vec![create_newline_trivia(ctx)];
+    let function_definition_trivia = vec![create_function_definition_trivia(ctx)];
 
+    #[cfg(feature = "luau")]
+    let attributes = local_function
+        .attributes()
+        .map(|attribute| {
+            format_luau_attribute(ctx, attribute, shape)
+                .update_leading_trivia(FormatTriviaType::Append(leading_trivia.clone()))
+                .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia.clone()))
+        })
+        .collect();
     let local_token = fmt_symbol!(ctx, local_function.local_token(), "local ", shape)
         .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
     let function_token = fmt_symbol!(ctx, local_function.function_token(), "function ", shape);
-    let formatted_name = format_token_reference(ctx, local_function.name(), shape);
+    let formatted_name = format_token_reference(ctx, local_function.name(), shape)
+        .update_trailing_trivia(FormatTriviaType::Append(function_definition_trivia));
 
     let shape = shape + (6 + 9 + strip_trivia(&formatted_name).to_string().len()); // 6 = "local ", 9 = "function "
     let function_body = format_function_body(ctx, local_function.body(), shape)
         .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia));
 
-    LocalFunction::new(formatted_name)
+    let local_function = LocalFunction::new(formatted_name)
         .with_local_token(local_token)
         .with_function_token(function_token)
-        .with_body(function_body)
+        .with_body(function_body);
+
+    #[cfg(feature = "luau")]
+    let local_function = local_function.with_attributes(attributes);
+
+    local_function
 }
 
 /// Formats a MethodCall node
@@ -1209,20 +1288,22 @@ pub fn format_method_call(
     shape: Shape,
     call_next_node: FunctionCallNextNode,
 ) -> MethodCall {
-    let formatted_colon_token = format_token_reference(ctx, method_call.colon_token(), shape);
-    let formatted_name = format_token_reference(ctx, method_call.name(), shape);
-    let shape =
-        shape + (formatted_colon_token.to_string().len() + formatted_name.to_string().len());
-    let formatted_function_args =
-        format_function_args(ctx, method_call.args(), shape, call_next_node);
+    let function_call_trivia = vec![create_function_call_trivia(ctx)];
 
-    MethodCall::new(formatted_name, formatted_function_args).with_colon_token(formatted_colon_token)
+    let (colon_token, name) =
+        process_dot_name(ctx, method_call.colon_token(), method_call.name(), shape);
+    let shape = shape + (colon_token.to_string().len() + name.to_string().len());
+    let formatted_function_args =
+        format_function_args(ctx, method_call.args(), shape, call_next_node)
+            .update_leading_trivia(FormatTriviaType::Append(function_call_trivia));
+
+    MethodCall::new(name, formatted_function_args).with_colon_token(colon_token)
 }
 
 /// Formats a single Parameter node
 pub fn format_parameter(ctx: &Context, parameter: &Parameter, shape: Shape) -> Parameter {
     match parameter {
-        Parameter::Ellipse(token) => Parameter::Ellipse(fmt_symbol!(ctx, token, "...", shape)),
+        Parameter::Ellipsis(token) => Parameter::Ellipsis(fmt_symbol!(ctx, token, "...", shape)),
         Parameter::Name(token_reference) => {
             Parameter::Name(format_token_reference(ctx, token_reference, shape))
         }
