@@ -1,3 +1,5 @@
+#[cfg(feature = "cfxlua")]
+use crate::LuaVersion;
 use crate::{
     context::{
         create_indent_trivia, create_newline_trivia, line_ending_character, Context, FormatNode,
@@ -5,19 +7,22 @@ use crate::{
     formatters::{
         trivia::{FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia, UpdateTrivia},
         trivia_util::{
-            self, punctuated_inline_comments, take_trailing_comments, GetLeadingTrivia,
-            GetTrailingTrivia, HasInlineComments,
+            self, punctuated_inline_comments, CommentSearch, GetLeadingTrivia, GetTrailingTrivia,
+            HasInlineComments,
         },
     },
     shape::Shape,
     QuoteStyle,
 };
-use full_moon::ast::{
-    punctuated::{Pair, Punctuated},
-    span::ContainedSpan,
-};
 use full_moon::node::Node;
 use full_moon::tokenizer::{StringLiteralQuoteType, Token, TokenKind, TokenReference, TokenType};
+use full_moon::{
+    ast::{
+        punctuated::{Pair, Punctuated},
+        span::ContainedSpan,
+    },
+    ShortString,
+};
 
 #[derive(Debug, Clone, Copy)]
 pub enum FormatTokenType {
@@ -42,7 +47,7 @@ macro_rules! fmt_symbol {
         $crate::formatters::general::format_symbol(
             $ctx,
             $token,
-            &TokenReference::symbol($x).unwrap(),
+            &TokenReference::symbol_specific_lua_version($x, $ctx.config().syntax.into()).unwrap(),
             $shape,
         )
     };
@@ -91,6 +96,98 @@ pub fn trivia_to_vec(
     output
 }
 
+fn format_string_literal(
+    ctx: &Context,
+    literal: &ShortString,
+    multi_line_depth: &usize,
+    quote_type: &StringLiteralQuoteType,
+) -> TokenType {
+    // CfxLua: if we have a backtick string, don't mess with it
+    #[cfg(feature = "cfxlua")]
+    if ctx.config().syntax == LuaVersion::CfxLua
+        && matches!(quote_type, StringLiteralQuoteType::Backtick)
+    {
+        return TokenType::StringLiteral {
+            literal: literal.clone(),
+            multi_line_depth: *multi_line_depth,
+            quote_type: StringLiteralQuoteType::Backtick,
+        };
+    }
+
+    // If we have a brackets string, don't mess with it
+    if let StringLiteralQuoteType::Brackets = quote_type {
+        // Convert the string to the correct line endings, by first normalising to LF
+        // then converting LF to output
+        let literal = literal
+            .replace("\r\n", "\n")
+            .replace('\n', &line_ending_character(ctx.config().line_endings));
+
+        TokenType::StringLiteral {
+            literal: literal.into(),
+            multi_line_depth: *multi_line_depth,
+            quote_type: StringLiteralQuoteType::Brackets,
+        }
+    } else {
+        // Match all escapes within the string
+        // Based off https://github.com/prettier/prettier/blob/181a325c1c07f1a4f3738665b7b28288dfb960bc/src/common/util.js#L439
+        lazy_static::lazy_static! {
+            static ref RE: regex::Regex = regex::Regex::new(r#"\\?(["'])|\\([\S\s])"#).unwrap();
+            static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-9\\abfnrtuvxz]$"#).unwrap();
+        }
+        let quote_to_use = get_quote_to_use(ctx, literal);
+        let literal = RE
+            .replace_all(literal, |caps: &regex::Captures| {
+                let quote = caps.get(1);
+                let escaped = caps.get(2);
+
+                match quote {
+                    Some(quote) => {
+                        // We have a quote, find what type it is, and see if we need to escape it
+                        // then return the output string
+                        match quote.as_str() {
+                            "'" => {
+                                // Check whether to escape the quote
+                                if let StringLiteralQuoteType::Single = quote_to_use {
+                                    String::from("\\'")
+                                } else {
+                                    String::from("'")
+                                }
+                            }
+                            "\"" => {
+                                // Check whether to escape the quote
+                                if let StringLiteralQuoteType::Double = quote_to_use {
+                                    String::from("\\\"")
+                                } else {
+                                    String::from("\"")
+                                }
+                            }
+                            other => unreachable!("unknown quote type {:?}", other),
+                        }
+                    }
+                    None => {
+                        // We have a normal escape
+                        // Test to see if it is necessary, and if not, then unescape it
+                        let text = escaped
+                            .expect("have a match which was neither an escape or a quote")
+                            .as_str();
+                        if UNNECESSARY_ESCAPES.is_match(text) {
+                            text.to_owned()
+                        } else {
+                            format!("\\{}", text.to_owned())
+                        }
+                    }
+                }
+            })
+            .into();
+
+        TokenType::StringLiteral {
+            literal,
+            multi_line_depth: *multi_line_depth,
+            quote_type: quote_to_use,
+        }
+    }
+}
+
 /// Formats a Token Node
 /// Also returns any extra leading or trailing trivia to add for the Token node
 /// This should only ever be called from format_token_reference
@@ -118,80 +215,17 @@ pub fn format_token(
         }
         TokenType::StringLiteral {
             literal,
-            multi_line,
+            multi_line_depth,
             quote_type,
-        } => {
-            // If we have a brackets string, don't mess with it
-            if let StringLiteralQuoteType::Brackets = quote_type {
-                // Convert the string to the correct line endings, by first normalising to LF
-                // then converting LF to output
-                let literal = literal
-                    .replace("\r\n", "\n")
-                    .replace('\n', &line_ending_character(ctx.config().line_endings));
+        } => format_string_literal(ctx, literal, multi_line_depth, quote_type),
+        TokenType::Shebang { line } => {
+            let line = format_single_line_comment_string(line).into();
 
-                TokenType::StringLiteral {
-                    literal: literal.into(),
-                    multi_line: *multi_line,
-                    quote_type: StringLiteralQuoteType::Brackets,
-                }
-            } else {
-                // Match all escapes within the string
-                // Based off https://github.com/prettier/prettier/blob/181a325c1c07f1a4f3738665b7b28288dfb960bc/src/common/util.js#L439
-                lazy_static::lazy_static! {
-                    static ref RE: regex::Regex = regex::Regex::new(r#"\\?(["'])|\\([\S\s])"#).unwrap();
-                    static ref UNNECESSARY_ESCAPES: regex::Regex = regex::Regex::new(r#"^[^\n\r"'0-9\\abfnrtuvxz]$"#).unwrap();
-                }
-                let quote_to_use = get_quote_to_use(ctx, literal);
-                let literal = RE
-                    .replace_all(literal, |caps: &regex::Captures| {
-                        let quote = caps.get(1);
-                        let escaped = caps.get(2);
+            // Shebang must always be leading trivia, as it is start of file. Terminate it with a newline
+            debug_assert!(matches!(format_type, FormatTokenType::LeadingTrivia));
+            trailing_trivia = Some(vec![create_newline_trivia(ctx)]);
 
-                        match quote {
-                            Some(quote) => {
-                                // We have a quote, find what type it is, and see if we need to escape it
-                                // then return the output string
-                                match quote.as_str() {
-                                    "'" => {
-                                        // Check whether to escape the quote
-                                        if let StringLiteralQuoteType::Single = quote_to_use {
-                                            String::from("\\'")
-                                        } else {
-                                            String::from("'")
-                                        }
-                                    }
-                                    "\"" => {
-                                        // Check whether to escape the quote
-                                        if let StringLiteralQuoteType::Double = quote_to_use {
-                                            String::from("\\\"")
-                                        } else {
-                                            String::from("\"")
-                                        }
-                                    }
-                                    other => unreachable!("unknown quote type {:?}", other),
-                                }
-                            }
-                            None => {
-                                // We have a normal escape
-                                // Test to see if it is necessary, and if not, then unescape it
-                                let text = escaped
-                                    .expect("have a match which was neither an escape or a quote")
-                                    .as_str();
-                                if UNNECESSARY_ESCAPES.is_match(text) {
-                                    text.to_owned()
-                                } else {
-                                    format!("\\{}", text.to_owned())
-                                }
-                            }
-                        }
-                    })
-                    .into();
-                TokenType::StringLiteral {
-                    literal,
-                    multi_line: None,
-                    quote_type: quote_to_use,
-                }
-            }
+            TokenType::Shebang { line }
         }
         TokenType::SingleLineComment { comment } => {
             let comment = format_single_line_comment_string(comment).into();
@@ -292,7 +326,9 @@ fn load_token_trivia(
                 // Move to next trivia
                 continue;
             }
-            TokenType::SingleLineComment { .. } | TokenType::MultiLineComment { .. } => {
+            TokenType::Shebang { .. }
+            | TokenType::SingleLineComment { .. }
+            | TokenType::MultiLineComment { .. } => {
                 // If we have a comment, when `format_token` is called, it will put a newline at the end
                 // If this happens, we want to skip the next iteration if its a newline, as that has already been covered here
                 if let FormatTokenType::LeadingTrivia = format_token_type {
@@ -522,9 +558,15 @@ where
                 ctx, shape,
             )]));
 
-        // Take any trailing trivia (i.e. comments) from the argument, and append it to the end of the punctuation
-        let (formatted_argument, mut trailing_comments) =
-            take_trailing_comments(&formatted_argument);
+        // Any singleline comments must be moved to after the punctuation
+        // We should keep multiline comments in the same location
+        let multiline_comments =
+            formatted_argument.trailing_comments_search(CommentSearch::Multiline);
+        let singleline_comments =
+            formatted_argument.trailing_comments_search(CommentSearch::Single);
+
+        let formatted_argument = formatted_argument
+            .update_trailing_trivia(FormatTriviaType::Replace(multiline_comments));
 
         let punctuation = match argument.punctuation() {
             Some(punctuation) => {
@@ -544,8 +586,8 @@ where
                             x,
                         ]
                     })
+                    .chain(singleline_comments)
                     .collect();
-                trailing_trivia.append(&mut trailing_comments);
                 trailing_trivia.push(create_newline_trivia(ctx));
 
                 let symbol = symbol.update_trivia(
@@ -559,7 +601,7 @@ where
             // We need to do this because in function declarations, we format parameters but if they have a type
             // specifier we don't have access to put it after the type specifier
             None => Some(TokenReference::new(
-                trailing_comments,
+                singleline_comments,
                 create_newline_trivia(ctx),
                 vec![],
             )),
@@ -637,7 +679,7 @@ pub fn format_end_token(
     shape: Shape,
 ) -> TokenReference {
     // Indent any comments leading a token, as these comments are technically part of the function body block
-    let formatted_leading_trivia: Vec<Token> = load_token_trivia(
+    let mut formatted_leading_trivia: Vec<Token> = load_token_trivia(
         ctx,
         current_token.leading_trivia().collect(),
         FormatTokenType::LeadingTrivia,
@@ -656,37 +698,39 @@ pub fn format_end_token(
         shape,
     );
 
-    // Special case for block end tokens:
-    // We will reverse the leading trivia, and keep removing any newlines we find, until we find something else, then we stop.
-    // This is to remove unnecessary newlines at the end of the block.
-    let mut iter = formatted_leading_trivia.iter().rev().peekable();
+    if !ctx.should_preserve_trailing_block_newline_gaps() {
+        // Special case for block end tokens:
+        // We will reverse the leading trivia, and keep removing any newlines we find, until we find something else, then we stop.
+        // This is to remove unnecessary newlines at the end of the block.
+        let original_leading_trivia = std::mem::take(&mut formatted_leading_trivia);
+        let mut iter = original_leading_trivia.iter().cloned().rev().peekable();
 
-    let mut formatted_leading_trivia = Vec::new();
-    let mut stop_removal = false;
-    while let Some(x) = iter.next() {
-        match x.token_type() {
-            TokenType::Whitespace { ref characters } => {
-                if !stop_removal
-                    && characters.contains('\n')
-                    && !matches!(
-                        iter.peek().map(|x| x.token_kind()),
-                        Some(TokenKind::SingleLineComment) | Some(TokenKind::MultiLineComment)
-                    )
-                {
-                    continue;
-                } else {
+        let mut stop_removal = false;
+        while let Some(x) = iter.next() {
+            match x.token_type() {
+                TokenType::Whitespace { ref characters } => {
+                    if !stop_removal
+                        && characters.contains('\n')
+                        && !matches!(
+                            iter.peek().map(|x| x.token_kind()),
+                            Some(TokenKind::SingleLineComment) | Some(TokenKind::MultiLineComment)
+                        )
+                    {
+                        continue;
+                    } else {
+                        formatted_leading_trivia.push(x.to_owned());
+                    }
+                }
+                _ => {
                     formatted_leading_trivia.push(x.to_owned());
+                    stop_removal = true; // Stop removing newlines once we have seen some sort of comment
                 }
             }
-            _ => {
-                formatted_leading_trivia.push(x.to_owned());
-                stop_removal = true; // Stop removing newlines once we have seen some sort of comment
-            }
         }
-    }
 
-    // Need to reverse the vector since we reversed the iterator
-    formatted_leading_trivia.reverse();
+        // Need to reverse the vector since we reversed the iterator
+        formatted_leading_trivia.reverse();
+    }
 
     TokenReference::new(
         formatted_leading_trivia,
