@@ -1,9 +1,11 @@
-#[cfg(feature = "lua52")]
-use crate::formatters::lua52::{format_goto, format_goto_no_trivia, format_label};
+#[cfg(any(feature = "luau", feature = "cfxlua"))]
+use crate::formatters::compound_assignment::format_compound_assignment;
+#[cfg(any(feature = "lua52", feature = "luajit"))]
+use crate::formatters::goto::{format_goto, format_goto_no_trivia, format_label};
 #[cfg(feature = "luau")]
 use crate::formatters::luau::{
-    format_compound_assignment, format_exported_type_declaration, format_type_declaration_stmt,
-    format_type_specifier,
+    format_exported_type_declaration, format_exported_type_function, format_type_declaration_stmt,
+    format_type_function_stmt, format_type_specifier,
 };
 use crate::{
     context::{create_indent_trivia, create_newline_trivia, Context, FormatNode},
@@ -32,7 +34,7 @@ use crate::{
 use full_moon::{
     ast::{
         punctuated::Punctuated, Block, Call, Do, ElseIf, Expression, FunctionArgs, FunctionCall,
-        GenericFor, If, NumericFor, Repeat, Stmt, Suffix, Value, While,
+        GenericFor, If, NumericFor, Repeat, Stmt, Suffix, While,
     },
     tokenizer::{Token, TokenKind, TokenReference, TokenType},
 };
@@ -60,10 +62,6 @@ pub fn remove_condition_parentheses(expression: Expression) -> Expression {
             let (_, comments) = trivia_util::take_trailing_comments(&expression);
             inner_expression.update_trailing_trivia(FormatTriviaType::Append(comments))
         }
-        Expression::Value { value, .. } => match *value {
-            Value::ParenthesesExpression(expression) => remove_condition_parentheses(expression),
-            _ => expression,
-        },
         _ => expression,
     }
 }
@@ -111,39 +109,30 @@ fn hug_generic_for(expressions: &Punctuated<Expression>) -> bool {
     }
 
     match expression {
-        Expression::Value { value, .. } => {
-            match &**value {
-                // Ensure is function call
-                Value::FunctionCall(function_call) => {
-                    let mut suffixes = function_call.suffixes();
-                    // Test next 2 available suffixes
-                    match (suffixes.next(), suffixes.next()) {
-                        // Ensure at least one suffix, and only one suffix
-                        (Some(suffix), None) => match suffix {
-                            // Ensure suffix is a call with a single table constructor as argument
-                            Suffix::Call(Call::AnonymousCall(FunctionArgs::TableConstructor(
-                                _,
-                            ))) => true,
-                            Suffix::Call(Call::AnonymousCall(FunctionArgs::Parentheses {
-                                arguments,
-                                ..
-                            })) => {
-                                let mut arguments = arguments.iter();
-                                // Test next 2 available arguments
-                                match (arguments.next(), arguments.next()) {
-                                    // Ensure at least one argument, and only one argument
-                                    // And that the argument is a table constructor
-                                    (Some(Expression::Value { value, .. }), None) => {
-                                        matches!(**value, Value::TableConstructor(_))
-                                    }
-                                    _ => false,
-                                }
-                            }
+        // Ensure is function call
+        Expression::FunctionCall(function_call) => {
+            let mut suffixes = function_call.suffixes();
+            // Test next 2 available suffixes
+            match (suffixes.next(), suffixes.next()) {
+                // Ensure at least one suffix, and only one suffix
+                (Some(suffix), None) => match suffix {
+                    // Ensure suffix is a call with a single table constructor as argument
+                    Suffix::Call(Call::AnonymousCall(FunctionArgs::TableConstructor(_))) => true,
+                    Suffix::Call(Call::AnonymousCall(FunctionArgs::Parentheses {
+                        arguments,
+                        ..
+                    })) => {
+                        let mut arguments = arguments.iter();
+                        // Test next 2 available arguments
+                        match (arguments.next(), arguments.next()) {
+                            // Ensure at least one argument, and only one argument
+                            // And that the argument is a table constructor
+                            (Some(Expression::TableConstructor(_)), None) => true,
                             _ => false,
-                        },
-                        _ => false,
+                        }
                     }
-                }
+                    _ => false,
+                },
                 _ => false,
             }
         }
@@ -426,6 +415,7 @@ fn format_else_if(ctx: &Context, else_if_node: &ElseIf, shape: Shape) -> ElseIf
 /// - There is no elseif/else block
 /// - The body of the if block only contains a single [`LastStmt`] (note, this LastStmt may have multiple expressions e.g. `return foo, bar`)
 /// - There are no internal comments within the block
+///
 /// We will also check if the statement surpasses the column width, or if the condition required multilining (handled outside of this function)
 fn is_if_guard(if_node: &If) -> bool {
     if_node.else_if().is_none()
@@ -468,8 +458,12 @@ pub fn format_if(ctx: &Context, if_node: &If, shape: Shape) -> If {
         // Rather than deferring to `format_block()`, since we know that there is only a single Stmt or LastStmt in the block, we can format it immediately
         // We need to modify the formatted LastStmt, since it will have automatically added leading/trailing trivia we don't want
         // We assume that there is only a laststmt present in the block - the callee of this function should have already checked for this
-        let stmt_leading_trivia = FormatTriviaType::Append(vec![Token::new(TokenType::spaces(1))]);
-        let stmt_trailing_trivia = FormatTriviaType::Append(vec![Token::new(TokenType::spaces(1))]);
+        // INVARIANT: this stmt has no leading/trailing comments, as this is checked in `is_if_guard`
+        // This means we can replace trivia completely
+        debug_assert!(!trivia_util::contains_comments(if_node.block()));
+        let stmt_leading_trivia = FormatTriviaType::Replace(vec![Token::new(TokenType::spaces(1))]);
+        let stmt_trailing_trivia =
+            FormatTriviaType::Replace(vec![Token::new(TokenType::spaces(1))]);
 
         let block = if let Some(stmt) = if_node.block().stmts().next() {
             let stmt = format_stmt_no_trivia(ctx, stmt, singleline_shape)
@@ -801,9 +795,11 @@ pub fn format_function_call_stmt(
 /// These are used for range formatting
 pub(crate) mod stmt_block {
     use crate::{context::Context, formatters::block::format_block, shape::Shape};
+    #[cfg(feature = "luau")]
+    use full_moon::ast::luau::TypeFunction;
     use full_moon::ast::{
         Call, Expression, Field, FunctionArgs, FunctionCall, Index, Prefix, Stmt, Suffix,
-        TableConstructor, Value,
+        TableConstructor,
     };
 
     fn format_table_constructor_block(
@@ -832,6 +828,8 @@ pub(crate) mod stmt_block {
                         equal,
                         value: format_expression_block(ctx, &value, shape),
                     },
+                    #[cfg(feature = "cfxlua")]
+                    Field::SetConstructor { dot, name } => Field::SetConstructor { dot, name },
                     Field::NoKey(expression) => {
                         Field::NoKey(format_expression_block(ctx, &expression, shape))
                     }
@@ -876,7 +874,7 @@ pub(crate) mod stmt_block {
     ) -> FunctionCall {
         let prefix = match function_call.prefix() {
             Prefix::Expression(expression) => {
-                Prefix::Expression(format_expression_block(ctx, expression, shape))
+                Prefix::Expression(Box::new(format_expression_block(ctx, expression, shape)))
             }
             Prefix::Name(name) => Prefix::Name(name.to_owned()),
             other => panic!("unknown node {:?}", other),
@@ -915,6 +913,17 @@ pub(crate) mod stmt_block {
             .with_suffixes(suffixes)
     }
 
+    #[cfg(feature = "luau")]
+    fn format_type_function_block(
+        ctx: &Context,
+        type_function: &TypeFunction,
+        shape: Shape,
+    ) -> TypeFunction {
+        let block = format_block(ctx, type_function.function_body().block(), shape);
+        let body = type_function.function_body().to_owned().with_block(block);
+        type_function.to_owned().with_function_body(body)
+    }
+
     /// Only formats a block within an expression
     pub fn format_expression_block(
         ctx: &Context,
@@ -938,35 +947,27 @@ pub(crate) mod stmt_block {
                 unop: unop.to_owned(),
                 expression: Box::new(format_expression_block(ctx, expression, shape)),
             },
-            Expression::Value {
-                value,
-                #[cfg(feature = "luau")]
+            Expression::Function(anonymous_function) => {
+                let block = format_block(ctx, anonymous_function.body().block(), shape);
+                let body = anonymous_function.body().clone().with_block(block);
+                Expression::Function(Box::new(anonymous_function.clone().with_body(body)))
+            }
+            Expression::FunctionCall(function_call) => {
+                Expression::FunctionCall(format_function_call_block(ctx, function_call, shape))
+            }
+            Expression::TableConstructor(table_constructor) => Expression::TableConstructor(
+                format_table_constructor_block(ctx, table_constructor, shape),
+            ),
+            #[cfg(feature = "luau")]
+            Expression::TypeAssertion {
+                expression,
                 type_assertion,
-            } => Expression::Value {
-                value: Box::new(match &**value {
-                    Value::Function((function_token, body)) => {
-                        let block = format_block(ctx, body.block(), shape);
-                        Value::Function((
-                            function_token.to_owned(),
-                            body.to_owned().with_block(block),
-                        ))
-                    }
-                    Value::FunctionCall(function_call) => {
-                        Value::FunctionCall(format_function_call_block(ctx, function_call, shape))
-                    }
-                    Value::TableConstructor(table_constructor) => Value::TableConstructor(
-                        format_table_constructor_block(ctx, table_constructor, shape),
-                    ),
-                    Value::ParenthesesExpression(expression) => Value::ParenthesesExpression(
-                        format_expression_block(ctx, expression, shape),
-                    ),
-                    // TODO: var?
-                    value => value.to_owned(),
-                }),
-                #[cfg(feature = "luau")]
+            } => Expression::TypeAssertion {
+                expression: Box::new(format_expression_block(ctx, expression, shape)),
                 type_assertion: type_assertion.to_owned(),
             },
-            other => panic!("unknown node {:?}", other),
+            // TODO: var?
+            value => value.to_owned(),
         }
     }
 
@@ -1062,7 +1063,7 @@ pub(crate) mod stmt_block {
                 let block = format_block(ctx, while_block.block(), block_shape);
                 Stmt::While(while_block.to_owned().with_block(block))
             }
-            #[cfg(feature = "luau")]
+            #[cfg(any(feature = "luau", feature = "cfxlua"))]
             Stmt::CompoundAssignment(compound_assignment) => {
                 let rhs = format_expression_block(ctx, compound_assignment.rhs(), block_shape);
                 Stmt::CompoundAssignment(compound_assignment.to_owned().with_rhs(rhs))
@@ -1071,9 +1072,26 @@ pub(crate) mod stmt_block {
             Stmt::ExportedTypeDeclaration(node) => Stmt::ExportedTypeDeclaration(node.to_owned()),
             #[cfg(feature = "luau")]
             Stmt::TypeDeclaration(node) => Stmt::TypeDeclaration(node.to_owned()),
-            #[cfg(feature = "lua52")]
+            #[cfg(feature = "luau")]
+            Stmt::ExportedTypeFunction(exported_type_function) => {
+                let type_function = format_type_function_block(
+                    ctx,
+                    exported_type_function.type_function(),
+                    block_shape,
+                );
+                Stmt::ExportedTypeFunction(
+                    exported_type_function
+                        .to_owned()
+                        .with_type_function(type_function),
+                )
+            }
+            #[cfg(feature = "luau")]
+            Stmt::TypeFunction(type_function) => {
+                Stmt::TypeFunction(format_type_function_block(ctx, type_function, block_shape))
+            }
+            #[cfg(any(feature = "lua52", feature = "luajit"))]
             Stmt::Goto(node) => Stmt::Goto(node.to_owned()),
-            #[cfg(feature = "lua52")]
+            #[cfg(any(feature = "lua52", feature = "luajit"))]
             Stmt::Label(node) => Stmt::Label(node.to_owned()),
             other => panic!("unknown node {:?}", other),
         }
@@ -1101,11 +1119,13 @@ pub fn format_stmt(ctx: &Context, stmt: &Stmt, shape: Shape) -> Stmt {
         NumericFor = format_numeric_for,
         Repeat = format_repeat_block,
         While = format_while_block,
-        #[cfg(feature = "luau")] CompoundAssignment = format_compound_assignment,
+        #[cfg(any(feature = "luau", feature = "cfxlua"))] CompoundAssignment = format_compound_assignment,
         #[cfg(feature = "luau")] ExportedTypeDeclaration = format_exported_type_declaration,
         #[cfg(feature = "luau")] TypeDeclaration = format_type_declaration_stmt,
-        #[cfg(feature = "lua52")] Goto = format_goto,
-        #[cfg(feature = "lua52")] Label = format_label,
+        #[cfg(feature = "luau")] ExportedTypeFunction = format_exported_type_function,
+        #[cfg(feature = "luau")] TypeFunction = format_type_function_stmt,
+        #[cfg(any(feature = "lua52", feature = "luajit"))] Goto = format_goto,
+        #[cfg(any(feature = "lua52", feature = "luajit"))] Label = format_label,
     })
 }
 
@@ -1121,7 +1141,7 @@ pub fn format_stmt_no_trivia(ctx: &Context, stmt: &Stmt, shape: Shape) -> Stmt {
         }
         Stmt::Assignment(stmt) => Stmt::Assignment(format_assignment_no_trivia(ctx, stmt, shape)),
         Stmt::FunctionCall(stmt) => Stmt::FunctionCall(format_function_call(ctx, stmt, shape)),
-        #[cfg(feature = "lua52")]
+        #[cfg(any(feature = "lua52", feature = "luajit"))]
         Stmt::Goto(goto) => Stmt::Goto(format_goto_no_trivia(ctx, goto, shape)),
         _ => unreachable!("format_stmt_no_trivia: node != assignment/function call/goto"),
     }
