@@ -9,7 +9,7 @@ use crate::{
             trivia_to_vec, EndTokenType, FormatTokenType,
         },
         trivia::{strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia},
-        trivia_util::{self, GetTrailingTrivia, HasInlineComments},
+        trivia_util::{self, CommentSearch, GetTrailingTrivia, HasInlineComments},
     },
     shape::Shape,
 };
@@ -17,12 +17,14 @@ use full_moon::{
     ast::{
         punctuated::{Pair, Punctuated},
         span::ContainedSpan,
-        Expression, Field, TableConstructor, Value,
+        Expression, Field, TableConstructor,
     },
     node::Node,
     tokenizer::{Token, TokenReference, TokenType},
 };
 
+use super::{assignment::hang_at_equals_due_to_comments, trivia_util::GetLeadingTrivia};
+
 /// Used to provide information about the table
 #[derive(Debug, Clone, Copy)]
 pub enum TableType {
@@ -40,17 +42,19 @@ fn format_field_expression_value(
     expression: &Expression,
     shape: Shape,
 ) -> Expression {
-    // Remove all trivia from the output expression as it will be moved after the comma
+    // Remove singleline comments from the output expression as it will be moved after the comma
+    // Retain multiline comments in place
+    let multiline_comments = expression.trailing_comments_search(CommentSearch::Multiline);
+    let trailing_trivia = FormatTriviaType::Replace(multiline_comments);
 
     if trivia_util::can_hang_expression(expression) {
         if expression.has_inline_comments() {
-            hang_expression(ctx, expression, shape, Some(1))
-                .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+            hang_expression(ctx, expression, shape, Some(1)).update_trailing_trivia(trailing_trivia)
         } else {
             let singleline_value = format_expression(ctx, expression, shape)
-                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+                .update_trailing_trivia(trailing_trivia.clone());
             let hanging_value = hang_expression(ctx, expression, shape, Some(1))
-                .update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+                .update_trailing_trivia(trailing_trivia);
 
             if shape.test_over_budget(&singleline_value)
                 || format!("{hanging_value}").lines().count()
@@ -62,11 +66,29 @@ fn format_field_expression_value(
             }
         }
     } else {
-        format_expression(ctx, expression, shape)
-            .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+        format_expression(ctx, expression, shape).update_trailing_trivia(trailing_trivia)
     }
 }
 
+fn hang_field_value_at_equals_due_to_comments(
+    ctx: &Context,
+    equal: &TokenReference,
+    expression: &Expression,
+    shape: Shape,
+) -> (TokenReference, Expression) {
+    // Remove singleline comments from the output expression as it will be moved after the comma
+    // Retain multiline comments in place
+    let multiline_comments = expression.trailing_comments_search(CommentSearch::Multiline);
+    let trailing_trivia = FormatTriviaType::Replace(multiline_comments);
+
+    let (equal_token, expression) = hang_at_equals_due_to_comments(ctx, equal, expression, shape);
+
+    (
+        equal_token,
+        expression.update_trailing_trivia(trailing_trivia),
+    )
+}
+
 /// Handles the formatting of the comments around a key and the equals sign in a field of a table.
 /// Takes in the key as a node (so that we can handle both expression key brackets and name keys)
 /// as well as the equals sign, then outputs the new leading trivia of the key + new equals token. The trailing trivia of the key should be emptied.
@@ -139,6 +161,8 @@ fn format_field(
         _ => FormatTriviaType::NoChange,
     };
 
+    // Trailing trivia is taken out and moved to after the comma
+    // We only move singleline comments, multiline comments remain in place
     let trailing_trivia;
     let field = match field {
         Field::ExpressionKey {
@@ -147,7 +171,7 @@ fn format_field(
             equal,
             value,
         } => {
-            trailing_trivia = value.trailing_trivia();
+            trailing_trivia = value.trailing_comments_search(CommentSearch::Single);
             let brackets = format_contained_span(ctx, brackets, shape);
 
             let space_brackets = is_brackets_string(key);
@@ -174,8 +198,14 @@ fn format_field(
                 .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
                 .update_leading_trivia(leading_trivia);
 
-            let shape = shape.take_last_line(&key) + (2 + 3 + if space_brackets { 2 } else { 0 }); // 2 = brackets, 3 = " = ", 2 = spaces around brackets if necessary
-            let value = format_field_expression_value(ctx, value, shape);
+            let (equal, value) = if value.has_leading_comments(CommentSearch::Single) {
+                hang_field_value_at_equals_due_to_comments(ctx, &equal, value, shape)
+            } else {
+                let shape =
+                    shape.take_last_line(&key) + (2 + 3 + if space_brackets { 2 } else { 0 }); // 2 = brackets, 3 = " = ", 2 = spaces around brackets if necessary
+                let value = format_field_expression_value(ctx, value, shape);
+                (equal, value)
+            };
 
             Field::ExpressionKey {
                 brackets,
@@ -185,7 +215,7 @@ fn format_field(
             }
         }
         Field::NameKey { key, equal, value } => {
-            trailing_trivia = value.trailing_trivia();
+            trailing_trivia = value.trailing_comments_search(CommentSearch::Single);
             let key = format_token_reference(ctx, key, shape);
 
             // Get the new leading comments to add before the key, and the equal token
@@ -199,13 +229,26 @@ fn format_field(
                 .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
                 .update_leading_trivia(leading_trivia);
 
-            let shape = shape + (strip_trivia(&key).to_string().len() + 3); // 3 = " = "
-            let value = format_field_expression_value(ctx, value, shape);
+            let (equal, value) = if value.has_leading_comments(CommentSearch::Single) {
+                hang_field_value_at_equals_due_to_comments(ctx, &equal, value, shape)
+            } else {
+                let shape = shape + (strip_trivia(&key).to_string().len() + 3); // 3 = " = "
+                let value = format_field_expression_value(ctx, value, shape);
+                (equal, value)
+            };
 
             Field::NameKey { key, equal, value }
         }
+        #[cfg(feature = "cfxlua")]
+        Field::SetConstructor { dot, name } => {
+            trailing_trivia = name.trailing_comments_search(CommentSearch::Single);
+            let dot = fmt_symbol!(ctx, dot, ".", shape);
+            let name = format_token_reference(ctx, name, shape);
+
+            Field::SetConstructor { dot, name }
+        }
         Field::NoKey(expression) => {
-            trailing_trivia = expression.trailing_trivia();
+            trailing_trivia = expression.trailing_comments_search(CommentSearch::Single);
 
             if let TableType::MultiLine = table_type {
                 let formatted_expression = format_field_expression_value(ctx, expression, shape);
@@ -305,7 +348,8 @@ where
 
         // Format the field. We will ignore the taken trailing trivia, as we do not need it.
         // (If there were any comments present, this function should never have been called)
-        let formatted_field = formatter(ctx, field, table_type, shape).0;
+        let (formatted_field, trailing_trivia) = formatter(ctx, field, table_type, shape);
+        assert!(trailing_trivia.is_empty());
 
         let formatted_punctuation = match current_fields.peek() {
             Some(_) => {
@@ -337,7 +381,7 @@ pub fn format_multiline_table<T, U>(
     shape: Shape,
 ) -> (ContainedSpan, Punctuated<T>)
 where
-    T: std::fmt::Display,
+    T: std::fmt::Display + Node,
     U: Fn(&Context, &T, TableType, Shape) -> (T, Vec<Token>),
 {
     let table_type = TableType::MultiLine;
@@ -349,14 +393,18 @@ where
     let current_fields = fields.pairs();
     let mut fields = Punctuated::new();
 
+    let mut ctx = *ctx;
+
     for pair in current_fields {
         let (field, punctuation) = (pair.value(), pair.punctuation());
 
+        ctx = ctx.check_toggle_formatting(field);
+
         // Reset the shape onto a new line, as we are a new field
         shape = shape.reset().add_width(1); // Add 1 to include the trailing comma at the end
 
         // Format the field
-        let (formatted_field, mut trailing_trivia) = formatter(ctx, field, table_type, shape);
+        let (formatted_field, mut trailing_trivia) = formatter(&ctx, field, table_type, shape);
 
         // If trivia is just whitespace, ignore it completely
         if trailing_trivia
@@ -366,21 +414,27 @@ where
             trailing_trivia = Vec::new();
         } else {
             // Filter trailing trivia for any newlines
+            // NOTE: in practice, this should only consist of singleline comments
             trailing_trivia = trailing_trivia
                 .iter()
                 .filter(|x| !trivia_util::trivia_is_whitespace(x))
                 .flat_map(|x| {
-                    trivia_to_vec(format_token(ctx, x, FormatTokenType::TrailingTrivia, shape))
+                    trivia_to_vec(format_token(
+                        &ctx,
+                        x,
+                        FormatTokenType::TrailingTrivia,
+                        shape,
+                    ))
                 })
                 .collect();
         }
 
         // Continue adding a comma and a new line for multiline tables
         // Add newline trivia to the end of the symbol
-        trailing_trivia.push(create_newline_trivia(ctx));
+        trailing_trivia.push(create_newline_trivia(&ctx));
 
         let symbol = match punctuation {
-            Some(punctuation) => fmt_symbol!(ctx, punctuation, ",", shape),
+            Some(punctuation) => fmt_symbol!(&ctx, punctuation, ",", shape),
             None => TokenReference::symbol(",").unwrap(),
         }
         .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia));
@@ -393,10 +447,8 @@ where
 }
 
 fn expression_is_multiline_function(ctx: &Context, expression: &Expression) -> bool {
-    if let Expression::Value { value, .. } = expression {
-        if let Value::Function((_, function_body)) = &**value {
-            return !should_collapse_function_body(ctx, function_body);
-        }
+    if let Expression::Function(anonymous_function) = expression {
+        return !should_collapse_function_body(ctx, anonymous_function.body());
     }
     false
 }
@@ -425,6 +477,8 @@ fn should_expand(ctx: &Context, table_constructor: &TableConstructor) -> bool {
                         || expression_is_multiline_function(ctx, value)
                 }
                 Field::NameKey { value, .. } => expression_is_multiline_function(ctx, value),
+                #[cfg(feature = "cfxlua")]
+                Field::SetConstructor { .. } => false,
                 Field::NoKey(expression) => expression_is_multiline_function(ctx, expression),
                 other => panic!("unknown node {:?}", other),
             };
