@@ -1,15 +1,16 @@
 #[cfg(feature = "lua54")]
 use full_moon::ast::lua54::Attribute;
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{
+use full_moon::ast::luau::{
     ElseIfExpression, GenericDeclaration, GenericDeclarationParameter, GenericParameterInfo,
-    IfExpression, IndexedTypeInfo, InterpolatedString, InterpolatedStringSegment, TypeArgument,
-    TypeAssertion, TypeDeclaration, TypeField, TypeFieldKey, TypeInfo, TypeSpecifier,
+    IfExpression, IndexedTypeInfo, InterpolatedString, InterpolatedStringSegment, LuauAttribute,
+    TypeArgument, TypeAssertion, TypeDeclaration, TypeField, TypeFieldKey, TypeFunction, TypeInfo,
+    TypeIntersection, TypeSpecifier, TypeUnion,
 };
 use full_moon::ast::{
-    punctuated::Punctuated, span::ContainedSpan, BinOp, Call, Expression, FunctionArgs,
-    FunctionBody, FunctionCall, FunctionName, Index, LastStmt, MethodCall, Parameter, Prefix,
-    Return, Stmt, Suffix, TableConstructor, UnOp, Value, Var, VarExpression,
+    punctuated::Punctuated, span::ContainedSpan, AnonymousFunction, BinOp, Call, Expression,
+    FunctionArgs, FunctionBody, FunctionCall, FunctionName, Index, LastStmt, MethodCall, Parameter,
+    Prefix, Return, Stmt, Suffix, TableConstructor, UnOp, Var, VarExpression,
 };
 use full_moon::ast::{Assignment, If, LocalAssignment};
 use full_moon::tokenizer::{Token, TokenReference};
@@ -223,6 +224,12 @@ define_update_trivia!(Call, |this, leading, trailing| {
     }
 });
 
+define_update_trivia!(AnonymousFunction, |this, leading, trailing| {
+    this.clone()
+        .with_function_token(this.function_token().update_leading_trivia(leading))
+        .with_body(this.body().update_trailing_trivia(trailing))
+});
+
 define_update_leading_trivia!(Expression, |this, leading| {
     match this {
         Expression::Parentheses {
@@ -241,13 +248,39 @@ define_update_leading_trivia!(Expression, |this, leading| {
             binop: binop.to_owned(),
             rhs: rhs.to_owned(),
         },
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
+        Expression::Function(anonymous_function) => {
+            Expression::Function(Box::new(anonymous_function.update_leading_trivia(leading)))
+        }
+        Expression::FunctionCall(function_call) => {
+            Expression::FunctionCall(function_call.update_leading_trivia(leading))
+        }
+        Expression::Number(token_reference) => {
+            Expression::Number(token_reference.update_leading_trivia(leading))
+        }
+        Expression::String(token_reference) => {
+            Expression::String(token_reference.update_leading_trivia(leading))
+        }
+        Expression::Symbol(token_reference) => {
+            Expression::Symbol(token_reference.update_leading_trivia(leading))
+        }
+        Expression::TableConstructor(table_constructor) => {
+            Expression::TableConstructor(table_constructor.update_leading_trivia(leading))
+        }
+        Expression::Var(var) => Expression::Var(var.update_leading_trivia(leading)),
+        #[cfg(feature = "luau")]
+        Expression::IfExpression(if_expression) => {
+            Expression::IfExpression(if_expression.update_leading_trivia(leading))
+        }
+        #[cfg(feature = "luau")]
+        Expression::InterpolatedString(interpolated_string) => {
+            Expression::InterpolatedString(interpolated_string.update_leading_trivia(leading))
+        }
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion {
+            expression,
             type_assertion,
-        } => Expression::Value {
-            value: Box::new(value.update_leading_trivia(leading)),
-            #[cfg(feature = "luau")]
+        } => Expression::TypeAssertion {
+            expression: Box::new(expression.update_leading_trivia(leading)),
             type_assertion: type_assertion.to_owned(),
         },
         other => panic!("unknown node {:?}", other),
@@ -256,26 +289,41 @@ define_update_leading_trivia!(Expression, |this, leading| {
 
 define_update_trailing_trivia!(Expression, |this, trailing| {
     match this {
-        Expression::Value {
-            value,
-            #[cfg(feature = "luau")]
-            type_assertion,
-        } => {
-            #[cfg(feature = "luau")]
-            if let Some(as_assertion) = type_assertion {
-                return Expression::Value {
-                    value: value.to_owned(),
-                    type_assertion: Some(as_assertion.update_trailing_trivia(trailing)),
-                };
-            }
-
-            Expression::Value {
-                value: Box::new(value.update_trailing_trivia(trailing)),
-                #[cfg(feature = "luau")]
-                type_assertion: type_assertion.to_owned(),
-            }
+        Expression::Function(anonymous_function) => Expression::Function(Box::new(
+            anonymous_function.update_trailing_trivia(trailing),
+        )),
+        Expression::FunctionCall(function_call) => {
+            Expression::FunctionCall(function_call.update_trailing_trivia(trailing))
         }
-
+        Expression::Number(token_reference) => {
+            Expression::Number(token_reference.update_trailing_trivia(trailing))
+        }
+        Expression::String(token_reference) => {
+            Expression::String(token_reference.update_trailing_trivia(trailing))
+        }
+        Expression::Symbol(token_reference) => {
+            Expression::Symbol(token_reference.update_trailing_trivia(trailing))
+        }
+        Expression::TableConstructor(table_constructor) => {
+            Expression::TableConstructor(table_constructor.update_trailing_trivia(trailing))
+        }
+        Expression::Var(var) => Expression::Var(var.update_trailing_trivia(trailing)),
+        #[cfg(feature = "luau")]
+        Expression::IfExpression(if_expression) => {
+            Expression::IfExpression(if_expression.update_trailing_trivia(trailing))
+        }
+        #[cfg(feature = "luau")]
+        Expression::InterpolatedString(interpolated_string) => {
+            Expression::InterpolatedString(interpolated_string.update_trailing_trivia(trailing))
+        }
+        #[cfg(feature = "luau")]
+        Expression::TypeAssertion {
+            expression,
+            type_assertion,
+        } => Expression::TypeAssertion {
+            expression: expression.to_owned(),
+            type_assertion: type_assertion.update_trailing_trivia(trailing),
+        },
         // Add trailing trivia to the end of parentheses
         Expression::Parentheses {
             contained,
@@ -407,7 +455,7 @@ define_update_trivia!(MethodCall, |this, leading, trailing| {
 
 define_update_trivia!(Parameter, |this, leading, trailing| {
     match this {
-        Parameter::Ellipse(token) => Parameter::Ellipse(token.update_trivia(leading, trailing)),
+        Parameter::Ellipsis(token) => Parameter::Ellipsis(token.update_trivia(leading, trailing)),
         Parameter::Name(token) => Parameter::Name(token.update_trivia(leading, trailing)),
         other => panic!("unknown node {:?}", other),
     }
@@ -419,7 +467,7 @@ define_update_leading_trivia!(Prefix, |this, leading| {
             Prefix::Name(token_reference.update_leading_trivia(leading))
         }
         Prefix::Expression(expression) => {
-            Prefix::Expression(expression.update_leading_trivia(leading))
+            Prefix::Expression(Box::new(expression.update_leading_trivia(leading)))
         }
         other => panic!("unknown node {:?}", other),
     }
@@ -431,7 +479,7 @@ define_update_trailing_trivia!(Prefix, |this, trailing| {
             Prefix::Name(token_reference.update_trailing_trivia(trailing))
         }
         Prefix::Expression(expression) => {
-            Prefix::Expression(expression.update_trailing_trivia(trailing))
+            Prefix::Expression(Box::new(expression.update_trailing_trivia(trailing)))
         }
         other => panic!("unknown node {:?}", other),
     }
@@ -636,13 +684,25 @@ define_update_trivia!(Stmt, |this, leading, trailing| {
         }
         #[cfg(feature = "luau")]
         Stmt::TypeDeclaration(stmt) => Stmt::TypeDeclaration(stmt.update_trivia(leading, trailing)),
-        #[cfg(feature = "lua52")]
+        #[cfg(feature = "luau")]
+        Stmt::ExportedTypeFunction(stmt) => {
+            let export_token = stmt.export_token().update_leading_trivia(leading);
+            let type_function = stmt.type_function().update_trailing_trivia(trailing);
+            Stmt::ExportedTypeFunction(
+                stmt.to_owned()
+                    .with_export_token(export_token)
+                    .with_type_function(type_function),
+            )
+        }
+        #[cfg(feature = "luau")]
+        Stmt::TypeFunction(stmt) => Stmt::TypeFunction(stmt.update_trivia(leading, trailing)),
+        #[cfg(any(feature = "lua52", feature = "luajit"))]
         Stmt::Goto(stmt) => Stmt::Goto(
             stmt.to_owned()
                 .with_goto_token(stmt.goto_token().update_leading_trivia(leading))
                 .with_label_name(stmt.label_name().update_trailing_trivia(trailing)),
         ),
-        #[cfg(feature = "lua52")]
+        #[cfg(any(feature = "lua52", feature = "luajit"))]
         Stmt::Label(stmt) => Stmt::Label(
             stmt.to_owned()
                 .with_left_colons(stmt.left_colons().update_leading_trivia(leading))
@@ -676,85 +736,11 @@ define_update_leading_trivia!(UnOp, |this, leading| {
     }
 });
 
-define_update_leading_trivia!(Value, |this, leading| {
-    match this {
-        Value::Function((token, function_body)) => Value::Function((
-            token.update_leading_trivia(leading),
-            function_body.to_owned(),
-        )),
-        Value::FunctionCall(function_call) => {
-            Value::FunctionCall(function_call.update_leading_trivia(leading))
-        }
-        Value::Number(token_reference) => {
-            Value::Number(token_reference.update_leading_trivia(leading))
-        }
-        Value::ParenthesesExpression(expression) => {
-            Value::ParenthesesExpression(expression.update_leading_trivia(leading))
-        }
-        Value::String(token_reference) => {
-            Value::String(token_reference.update_leading_trivia(leading))
-        }
-        Value::Symbol(token_reference) => {
-            Value::Symbol(token_reference.update_leading_trivia(leading))
-        }
-        Value::TableConstructor(table_constructor) => {
-            Value::TableConstructor(table_constructor.update_leading_trivia(leading))
-        }
-        Value::Var(var) => Value::Var(var.update_leading_trivia(leading)),
-        #[cfg(feature = "luau")]
-        Value::IfExpression(if_expression) => {
-            Value::IfExpression(if_expression.update_leading_trivia(leading))
-        }
-        #[cfg(feature = "luau")]
-        Value::InterpolatedString(interpolated_string) => {
-            Value::InterpolatedString(interpolated_string.update_leading_trivia(leading))
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-});
-
-define_update_trailing_trivia!(Value, |this, trailing| {
-    match this {
-        Value::Function((token, function_body)) => Value::Function((
-            token.to_owned(),
-            function_body.update_trailing_trivia(trailing),
-        )),
-        Value::FunctionCall(function_call) => {
-            Value::FunctionCall(function_call.update_trailing_trivia(trailing))
-        }
-        Value::Number(token_reference) => {
-            Value::Number(token_reference.update_trailing_trivia(trailing))
-        }
-        Value::ParenthesesExpression(expression) => {
-            Value::ParenthesesExpression(expression.update_trailing_trivia(trailing))
-        }
-        Value::String(token_reference) => {
-            Value::String(token_reference.update_trailing_trivia(trailing))
-        }
-        Value::Symbol(token_reference) => {
-            Value::Symbol(token_reference.update_trailing_trivia(trailing))
-        }
-        Value::TableConstructor(table_constructor) => {
-            Value::TableConstructor(table_constructor.update_trailing_trivia(trailing))
-        }
-        Value::Var(var) => Value::Var(var.update_trailing_trivia(trailing)),
-        #[cfg(feature = "luau")]
-        Value::IfExpression(if_expression) => {
-            Value::IfExpression(if_expression.update_trailing_trivia(trailing))
-        }
-        #[cfg(feature = "luau")]
-        Value::InterpolatedString(interpolated_string) => {
-            Value::InterpolatedString(interpolated_string.update_trailing_trivia(trailing))
-        }
-        other => panic!("unknown node {:?}", other),
-    }
-});
-
 define_update_leading_trivia!(Var, |this, leading| {
     match this {
         Var::Name(token_reference) => Var::Name(token_reference.update_leading_trivia(leading)),
         Var::Expression(var_expresion) => {
-            Var::Expression(var_expresion.update_leading_trivia(leading))
+            Var::Expression(Box::new(var_expresion.update_leading_trivia(leading)))
         }
         other => panic!("unknown node {:?}", other),
     }
@@ -764,7 +750,7 @@ define_update_trailing_trivia!(Var, |this, trailing| {
     match this {
         Var::Name(token_reference) => Var::Name(token_reference.update_trailing_trivia(trailing)),
         Var::Expression(var_expression) => {
-            Var::Expression(var_expression.update_trailing_trivia(trailing))
+            Var::Expression(Box::new(var_expression.update_trailing_trivia(trailing)))
         }
         other => panic!("unknown node {:?}", other),
     }
@@ -789,11 +775,50 @@ define_update_trivia!(VarExpression, |this, leading, trailing| {
     this.to_owned().with_prefix(prefix).with_suffixes(suffixes)
 });
 
+#[cfg(feature = "luau")]
+define_update_trivia!(TypeUnion, |this, leading, trailing| {
+    if let Some(leading_token) = this.leading() {
+        TypeUnion::new(
+            Some(leading_token.update_leading_trivia(leading)),
+            this.types().update_trailing_trivia(trailing),
+        )
+    } else {
+        TypeUnion::new(
+            None,
+            this.types()
+                .update_leading_trivia(leading)
+                .update_trailing_trivia(trailing),
+        )
+    }
+});
+
+#[cfg(feature = "luau")]
+define_update_trivia!(TypeIntersection, |this, leading, trailing| {
+    if let Some(leading_token) = this.leading() {
+        TypeIntersection::new(
+            Some(leading_token.update_leading_trivia(leading)),
+            this.types().update_trailing_trivia(trailing),
+        )
+    } else {
+        TypeIntersection::new(
+            None,
+            this.types()
+                .update_leading_trivia(leading)
+                .update_trailing_trivia(trailing),
+        )
+    }
+});
+
 #[cfg(feature = "luau")]
 define_update_trivia!(TypeInfo, |this, leading, trailing| {
     match this {
-        TypeInfo::Array { braces, type_info } => TypeInfo::Array {
+        TypeInfo::Array {
+            braces,
+            access,
+            type_info,
+        } => TypeInfo::Array {
             braces: braces.update_trivia(leading, trailing),
+            access: access.to_owned(),
             type_info: type_info.to_owned(),
         },
         TypeInfo::Basic(token_reference) => {
@@ -838,20 +863,14 @@ define_update_trivia!(TypeInfo, |this, leading, trailing| {
             generics: generics.to_owned(),
         },
 
-        TypeInfo::GenericPack { name, ellipse } => TypeInfo::GenericPack {
+        TypeInfo::GenericPack { name, ellipsis } => TypeInfo::GenericPack {
             name: name.update_leading_trivia(leading),
-            ellipse: ellipse.update_trailing_trivia(trailing),
+            ellipsis: ellipsis.update_trailing_trivia(trailing),
         },
 
-        TypeInfo::Intersection {
-            left,
-            ampersand,
-            right,
-        } => TypeInfo::Intersection {
-            left: Box::new(left.update_leading_trivia(leading)),
-            ampersand: ampersand.to_owned(),
-            right: Box::new(right.update_trailing_trivia(trailing)),
-        },
+        TypeInfo::Intersection(intersection) => {
+            TypeInfo::Intersection(intersection.update_trivia(leading, trailing))
+        }
 
         TypeInfo::Module {
             module,
@@ -891,19 +910,18 @@ define_update_trivia!(TypeInfo, |this, leading, trailing| {
             types: types.to_owned(),
         },
 
-        TypeInfo::Union { left, pipe, right } => TypeInfo::Union {
-            left: Box::new(left.update_leading_trivia(leading)),
-            pipe: pipe.to_owned(),
-            right: Box::new(right.update_trailing_trivia(trailing)),
-        },
+        TypeInfo::Union(union) => TypeInfo::Union(union.update_trivia(leading, trailing)),
 
-        TypeInfo::Variadic { ellipse, type_info } => TypeInfo::Variadic {
-            ellipse: ellipse.update_leading_trivia(leading),
+        TypeInfo::Variadic {
+            ellipsis,
+            type_info,
+        } => TypeInfo::Variadic {
+            ellipsis: ellipsis.update_leading_trivia(leading),
             type_info: Box::new(type_info.update_trailing_trivia(trailing)),
         },
 
-        TypeInfo::VariadicPack { ellipse, name } => TypeInfo::VariadicPack {
-            ellipse: ellipse.update_leading_trivia(leading),
+        TypeInfo::VariadicPack { ellipsis, name } => TypeInfo::VariadicPack {
+            ellipsis: ellipsis.update_leading_trivia(leading),
             name: name.update_trailing_trivia(trailing),
         },
 
@@ -918,6 +936,13 @@ define_update_trivia!(TypeDeclaration, |this, leading, trailing| {
         .with_type_definition(this.type_definition().update_trailing_trivia(trailing))
 });
 
+#[cfg(feature = "luau")]
+define_update_trivia!(TypeFunction, |this, leading, trailing| {
+    this.to_owned()
+        .with_type_token(this.type_token().update_leading_trivia(leading))
+        .with_function_body(this.function_body().update_trailing_trivia(trailing))
+});
+
 #[cfg(feature = "luau")]
 define_update_trailing_trivia!(IndexedTypeInfo, |this, trailing| {
     match this {
@@ -966,8 +991,13 @@ define_update_trivia!(TypeAssertion, |this, leading, trailing| {
 
 #[cfg(feature = "luau")]
 define_update_leading_trivia!(TypeField, |this, leading| {
-    this.to_owned()
-        .with_key(this.key().update_leading_trivia(leading))
+    if let Some(access) = this.access() {
+        this.to_owned()
+            .with_access(Some(access.update_leading_trivia(leading)))
+    } else {
+        this.to_owned()
+            .with_key(this.key().update_leading_trivia(leading))
+    }
 });
 
 #[cfg(feature = "luau")]
@@ -1006,9 +1036,9 @@ define_update_leading_trivia!(GenericDeclarationParameter, |this, leading| {
         GenericParameterInfo::Name(token) => {
             GenericParameterInfo::Name(token.update_leading_trivia(leading))
         }
-        GenericParameterInfo::Variadic { name, ellipse } => GenericParameterInfo::Variadic {
+        GenericParameterInfo::Variadic { name, ellipsis } => GenericParameterInfo::Variadic {
             name: name.update_leading_trivia(leading),
-            ellipse: ellipse.to_owned(),
+            ellipsis: ellipsis.to_owned(),
         },
         other => panic!("unknown node {:?}", other),
     };
@@ -1027,9 +1057,9 @@ define_update_trailing_trivia!(GenericDeclarationParameter, |this, trailing| {
             GenericParameterInfo::Name(token) => {
                 GenericParameterInfo::Name(token.update_trailing_trivia(trailing))
             }
-            GenericParameterInfo::Variadic { name, ellipse } => GenericParameterInfo::Variadic {
+            GenericParameterInfo::Variadic { name, ellipsis } => GenericParameterInfo::Variadic {
                 name: name.to_owned(),
-                ellipse: ellipse.update_trailing_trivia(trailing),
+                ellipsis: ellipsis.update_trailing_trivia(trailing),
             },
             other => panic!("unknown node {:?}", other),
         };
@@ -1084,3 +1114,10 @@ define_update_leading_trivia!(InterpolatedStringSegment, |this, leading| {
         expression: this.expression.to_owned(),
     }
 });
+
+#[cfg(feature = "luau")]
+define_update_trivia!(LuauAttribute, |this, leading, trailing| {
+    this.clone()
+        .with_at_sign(this.at_sign().update_leading_trivia(leading))
+        .with_name(this.name().update_trailing_trivia(trailing))
+});
