@@ -1,5 +1,7 @@
 use context::Context;
 use full_moon::ast::Ast;
+#[cfg(all(feature = "luau", any(feature = "lua52", feature = "lua53")))]
+use full_moon::tokenizer::{Symbol, TokenType};
 use serde::Deserialize;
 use thiserror::Error;
 #[cfg(all(target_arch = "wasm32", feature = "wasm-bindgen"))]
@@ -14,6 +16,63 @@ mod shape;
 mod sort_requires;
 mod verify_ast;
 
+/// The Lua syntax version to use
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq, Deserialize)]
+#[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
+#[cfg_attr(feature = "serialize", derive(serde::Serialize))]
+#[cfg_attr(feature = "fromstr", derive(strum::EnumString))]
+pub enum LuaVersion {
+    /// Parse all syntax versions at the same time. This allows most general usage.
+    /// For overlapping syntaxes (e.g., Lua5.2 label syntax and Luau type assertions), select a
+    /// specific syntax version
+    #[default]
+    All,
+    /// Parse Lua 5.1 code
+    Lua51,
+    /// Parse Lua 5.2 code
+    #[cfg(feature = "lua52")]
+    Lua52,
+    /// Parse Lua 5.3 code
+    #[cfg(feature = "lua53")]
+    Lua53,
+    /// Parse Lua 5.4 code
+    #[cfg(feature = "lua54")]
+    Lua54,
+    /// Parse Luau code
+    #[cfg(feature = "luau")]
+    Luau,
+    /// Parse LuaJIT code
+    #[cfg(feature = "luajit")]
+    LuaJIT,
+    /// Parse Cfx Lua code
+    #[cfg(feature = "cfxlua")]
+    CfxLua,
+}
+
+impl From<LuaVersion> for full_moon::LuaVersion {
+    fn from(val: LuaVersion) -> Self {
+        match val {
+            #[cfg(feature = "cfxlua")]
+            LuaVersion::All => full_moon::LuaVersion::new().with_cfxlua(),
+            #[cfg(not(feature = "cfxlua"))]
+            LuaVersion::All => full_moon::LuaVersion::new(),
+            LuaVersion::Lua51 => full_moon::LuaVersion::lua51(),
+            #[cfg(feature = "lua52")]
+            LuaVersion::Lua52 => full_moon::LuaVersion::lua52(),
+            #[cfg(feature = "lua53")]
+            LuaVersion::Lua53 => full_moon::LuaVersion::lua53(),
+            #[cfg(feature = "lua54")]
+            LuaVersion::Lua54 => full_moon::LuaVersion::lua54(),
+            #[cfg(feature = "luau")]
+            LuaVersion::Luau => full_moon::LuaVersion::luau(),
+            #[cfg(feature = "luajit")]
+            LuaVersion::LuaJIT => full_moon::LuaVersion::luajit(),
+            #[cfg(feature = "cfxlua")]
+            LuaVersion::CfxLua => full_moon::LuaVersion::cfxlua(),
+        }
+    }
+}
+
 /// The type of indents to use when indenting
 #[derive(Debug, Default, Copy, Clone, PartialEq, Eq, Deserialize)]
 #[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
@@ -73,6 +132,8 @@ pub enum CallParenType {
     NoSingleTable,
     /// Skip call parentheses when only a table or string argument is used.
     None,
+    /// Keep call parentheses based on its presence in input code.
+    Input,
 }
 
 /// What mode to use if we want to collapse simple functions / guard statements
@@ -92,14 +153,27 @@ pub enum CollapseSimpleStatement {
     Always,
 }
 
+/// If blocks should be allowed to have leading and trailing newline gaps.
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq, Deserialize)]
+#[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
+#[cfg_attr(feature = "serialize", derive(serde::Serialize))]
+#[cfg_attr(feature = "fromstr", derive(strum::EnumString))]
+pub enum BlockNewlineGaps {
+    /// Never allow leading or trailing newline gaps
+    #[default]
+    Never,
+    /// Preserve both leading and trailing newline gaps if present in input
+    Preserve,
+}
+
 /// An optional formatting range.
 /// If provided, only content within these boundaries (inclusive) will be formatted.
 /// Both boundaries are optional, and are given as byte offsets from the beginning of the file.
 #[derive(Debug, Copy, Clone, Deserialize)]
 #[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
 pub struct Range {
-    start: Option<usize>,
-    end: Option<usize>,
+    pub start: Option<usize>,
+    pub end: Option<usize>,
 }
 
 #[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
@@ -118,41 +192,68 @@ impl Range {
 #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
 pub struct SortRequiresConfig {
     /// Whether the sort requires codemod is enabled
-    enabled: bool,
+    pub enabled: bool,
 }
 
+#[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
 impl SortRequiresConfig {
+    pub fn new() -> Self {
+        SortRequiresConfig::default()
+    }
+    #[deprecated(since = "0.19.0", note = "access `.enabled` directly instead")]
+    #[cfg(not(all(target_arch = "wasm32", feature = "wasm-bindgen")))]
     pub fn enabled(&self) -> bool {
         self.enabled
     }
+    #[deprecated(since = "0.19.0", note = "modify `.enabled` directly instead")]
     pub fn set_enabled(&self, enabled: bool) -> Self {
         Self { enabled }
     }
 }
 
+/// When to use spaces after function names
+#[derive(Debug, Default, Copy, Clone, PartialEq, Eq, Deserialize)]
+#[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
+#[cfg_attr(feature = "serialize", derive(serde::Serialize))]
+#[cfg_attr(feature = "fromstr", derive(strum::EnumString))]
+pub enum SpaceAfterFunctionNames {
+    /// Never use spaces after function names.
+    #[default]
+    Never,
+    /// Use spaces after function names only for function definitions.
+    Definitions,
+    /// Use spaces after function names only for function calls.
+    Calls,
+    /// Use spaces after function names in definitions and calls.
+    Always,
+}
+
 /// The configuration to use when formatting.
 #[derive(Copy, Clone, Debug, Deserialize)]
 #[serde(default, deny_unknown_fields)]
 #[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
 #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
 pub struct Config {
+    /// The type of Lua syntax to parse.
+    pub syntax: LuaVersion,
     /// The approximate line length to use when printing the code.
     /// This is used as a guide to determine when to wrap lines, but note
     /// that this is not a hard upper bound.
-    column_width: usize,
+    pub column_width: usize,
     /// The type of line endings to use.
-    line_endings: LineEndings,
+    pub line_endings: LineEndings,
     /// The type of indents to use.
-    indent_type: IndentType,
+    pub indent_type: IndentType,
     /// The width of a single indentation level.
     /// If `indent_type` is set to [`IndentType::Spaces`], then this is the number of spaces to use.
     /// If `indent_type` is set to [`IndentType::Tabs`], then this is used as a heuristic to guide when to wrap lines.
-    indent_width: usize,
+    pub indent_width: usize,
     /// The style of quotes to use in string literals.
-    quote_style: QuoteStyle,
+    pub quote_style: QuoteStyle,
     /// Whether to omit parentheses around function calls which take a single string literal or table.
     /// This is added for adoption reasons only, and is not recommended for new work.
-    no_call_parentheses: bool,
+    #[deprecated(note = "use `call_parentheses` instead")]
+    pub no_call_parentheses: bool,
     /// When to use call parentheses.
     /// if call_parentheses is set to [`CallParenType::Always`] call parentheses is always applied.
     /// if call_parentheses is set to [`CallParenType::NoSingleTable`] call parentheses is omitted when
@@ -161,13 +262,23 @@ pub struct Config {
     /// function is called with only one table argument.
     /// if call_parentheses is set to [`CallParenType::None`] call parentheses is omitted when
     /// function is called with only one table or string argument (same as no_call_parentheses).
-    call_parentheses: CallParenType,
+    pub call_parentheses: CallParenType,
     /// Whether we should collapse simple structures like functions or guard statements
     /// if set to [`CollapseSimpleStatement::None`] structures are never collapsed.
     /// if set to [`CollapseSimpleStatement::FunctionOnly`] then simple functions (i.e., functions with a single laststmt) can be collapsed
-    collapse_simple_statement: CollapseSimpleStatement,
+    pub collapse_simple_statement: CollapseSimpleStatement,
+    /// Whether we should allow blocks to preserve leading and trailing newline gaps.
+    /// if set to [`BlockNewlineGaps::Never`] then newline gaps are never allowed at the start or end of blocks.
+    /// if set to [`BlockNewlineGaps::Preserve`] then newline gaps are preserved at the start and end of blocks.
+    pub block_newline_gaps: BlockNewlineGaps,
     /// Configuration for the sort requires codemod
-    sort_requires: SortRequiresConfig,
+    pub sort_requires: SortRequiresConfig,
+    /// Whether we should include a space between the function name and arguments.
+    /// * if space_after_function_names is set to [`SpaceAfterFunctionNames::Never`] a space is never used.
+    /// * if space_after_function_names is set to [`SpaceAfterFunctionNames::Definitions`] a space is used only for definitions.
+    /// * if space_after_function_names is set to [`SpaceAfterFunctionNames::Calls`] a space is used only for calls.
+    /// * if space_after_function_names is set to [`SpaceAfterFunctionNames::Always`] a space is used for both definitions and calls.
+    pub space_after_function_names: SpaceAfterFunctionNames,
 }
 
 #[cfg_attr(all(target_arch = "wasm32", feature = "wasm-bindgen"), wasm_bindgen)]
@@ -176,123 +287,13 @@ impl Config {
     pub fn new() -> Self {
         Config::default()
     }
-
-    /// Returns the current configured column width
-    pub fn column_width(&self) -> usize {
-        self.column_width
-    }
-
-    /// Returns the current configured line endings
-    pub fn line_endings(&self) -> LineEndings {
-        self.line_endings
-    }
-
-    /// Returns the current configured indent type
-    pub fn indent_type(&self) -> IndentType {
-        self.indent_type
-    }
-
-    /// Returns the current configured indent width
-    pub fn indent_width(&self) -> usize {
-        self.indent_width
-    }
-
-    /// Returns the current configured quote style
-    pub fn quote_style(&self) -> QuoteStyle {
-        self.quote_style
-    }
-
-    /// Returns the current configured call parentheses style
-    pub fn call_parentheses(&self) -> CallParenType {
-        self.call_parentheses
-    }
-
-    pub fn collapse_simple_statement(&self) -> CollapseSimpleStatement {
-        self.collapse_simple_statement
-    }
-
-    /// Returns the current sort requires codemod configuration
-    pub fn sort_requires(&self) -> SortRequiresConfig {
-        self.sort_requires
-    }
-
-    /// Returns a new config with the given column width
-    pub fn with_column_width(self, column_width: usize) -> Self {
-        Self {
-            column_width,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given line endings
-    pub fn with_line_endings(self, line_endings: LineEndings) -> Self {
-        Self {
-            line_endings,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given indent type
-    pub fn with_indent_type(self, indent_type: IndentType) -> Self {
-        Self {
-            indent_type,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given indent width
-    pub fn with_indent_width(self, indent_width: usize) -> Self {
-        Self {
-            indent_width,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given quote style
-    pub fn with_quote_style(self, quote_style: QuoteStyle) -> Self {
-        Self {
-            quote_style,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given value for `no_call_parentheses`
-    pub fn with_no_call_parentheses(self, no_call_parentheses: bool) -> Self {
-        Self {
-            no_call_parentheses,
-            ..self
-        }
-    }
-    /// Returns a new config with the given call parentheses type
-    pub fn with_call_parentheses(self, call_parentheses: CallParenType) -> Self {
-        Self {
-            call_parentheses,
-            ..self
-        }
-    }
-
-    pub fn with_collapse_simple_statement(
-        self,
-        collapse_simple_statement: CollapseSimpleStatement,
-    ) -> Self {
-        Self {
-            collapse_simple_statement,
-            ..self
-        }
-    }
-
-    /// Returns a new config with the given sort requires configuration
-    pub fn with_sort_requires(self, sort_requires: SortRequiresConfig) -> Self {
-        Self {
-            sort_requires,
-            ..self
-        }
-    }
 }
 
 impl Default for Config {
     fn default() -> Self {
+        #[allow(deprecated)]
         Self {
+            syntax: LuaVersion::default(),
             column_width: 120,
             line_endings: LineEndings::default(),
             indent_type: IndentType::default(),
@@ -302,6 +303,8 @@ impl Default for Config {
             call_parentheses: CallParenType::default(),
             collapse_simple_statement: CollapseSimpleStatement::default(),
             sort_requires: SortRequiresConfig::default(),
+            space_after_function_names: SpaceAfterFunctionNames::default(),
+            block_newline_gaps: BlockNewlineGaps::default(),
         }
     }
 }
@@ -316,15 +319,83 @@ pub enum OutputVerification {
     None,
 }
 
+#[cfg(all(feature = "luau", feature = "lua53"))]
+fn is_luau_and_lua53_conflict(error: &full_moon::Error) -> bool {
+    match error {
+        full_moon::Error::AstError(ast_error) => match ast_error.token().token_type() {
+            TokenType::Symbol {
+                symbol: Symbol::DoubleGreaterThan,
+            } => ast_error.error_message() == "expected '>' to close generic type parameter list",
+            _ => false,
+        },
+        _ => false,
+    }
+}
+
+#[cfg(all(feature = "luau", feature = "lua52"))]
+fn is_luau_and_lua52_conflict(error: &full_moon::Error) -> bool {
+    match error {
+        full_moon::Error::AstError(ast_error) => {
+            ast_error.error_message() == "expected label name after `::`"
+        }
+        _ => false,
+    }
+}
+
+fn print_full_moon_error(error: &full_moon::Error) -> String {
+    match error {
+        full_moon::Error::AstError(ast_error) => format!(
+            "unexpected token `{}` ({}:{} to {}:{}), {}",
+            ast_error.token(),
+            ast_error.range().0.line(),
+            ast_error.range().0.character(),
+            ast_error.range().1.line(),
+            ast_error.range().1.character(),
+            ast_error.error_message()
+        ),
+        full_moon::Error::TokenizerError(tokenizer_error) => tokenizer_error.to_string(),
+    }
+}
+
+fn print_full_moon_errors(errors: &[full_moon::Error]) -> String {
+    #[allow(unused_mut)]
+    let mut error_message = if errors.len() == 1 {
+        print_full_moon_error(errors.first().unwrap())
+    } else {
+        errors
+            .iter()
+            .map(|err| "\n - ".to_string() + &print_full_moon_error(err))
+            .collect::<String>()
+    };
+
+    #[cfg(all(feature = "luau", feature = "lua53"))]
+    {
+        let recommend_luau_syntax = errors.iter().any(is_luau_and_lua53_conflict);
+        if recommend_luau_syntax {
+            error_message += "\nhint: this looks like a conflict with Lua 5.3 and Luau generics syntax, add `syntax = \"Luau\"` to stylua.toml to resolve";
+        }
+    }
+
+    #[cfg(all(feature = "luau", feature = "lua52"))]
+    {
+        let recommend_lua52_syntax = errors.iter().any(is_luau_and_lua52_conflict);
+        if recommend_lua52_syntax {
+            error_message += "\nhint: this looks like a conflict with Luau and Lua 5.2 label syntax, add `syntax = \"Lua52\"` to stylua.toml to resolve";
+        }
+    }
+
+    error_message
+}
+
 /// A formatting error
 #[derive(Clone, Debug, Error)]
 pub enum Error {
     /// The input AST has a parsing error.
-    #[error("error parsing: {0}")]
-    ParseError(full_moon::Error),
+    #[error("error parsing: {}", print_full_moon_errors(.0))]
+    ParseError(Vec<full_moon::Error>),
     /// The output AST after formatting generated a parse error. This is a definite error.
-    #[error("INTERNAL ERROR: Output AST generated a syntax error. Please report this at https://github.com/johnnymorganz/stylua/issues\n{0}")]
-    VerificationAstError(full_moon::Error),
+    #[error("INTERNAL ERROR: Output AST generated a syntax error. Please report this at https://github.com/johnnymorganz/stylua/issues: {}", print_full_moon_errors(.0))]
+    VerificationAstError(Vec<full_moon::Error>),
     /// The output AST after formatting differs from the input AST.
     #[error("INTERNAL WARNING: Output AST may be different to input AST. Code correctness may have changed. Please examine the formatting diff and report any issues at https://github.com/johnnymorganz/stylua/issues")]
     VerificationAstDifference,
@@ -348,7 +419,7 @@ pub fn format_ast(
     let ctx = Context::new(config, range);
 
     // Perform require sorting beforehand if necessary
-    let input_ast = match config.sort_requires().enabled() {
+    let input_ast = match config.sort_requires.enabled {
         true => sort_requires::sort_requires(&ctx, input_ast),
         false => input_ast,
     };
@@ -358,13 +429,14 @@ pub fn format_ast(
 
     // If we are verifying, reparse the output then check it matches the original input
     if let Some(input_ast) = input_ast_for_verification {
-        let output = full_moon::print(&ast);
-        let reparsed_output = match full_moon::parse(&output) {
-            Ok(ast) => ast,
-            Err(error) => {
-                return Err(Error::VerificationAstError(error));
-            }
-        };
+        let output = ast.to_string();
+        let reparsed_output =
+            match full_moon::parse_fallible(&output, config.syntax.into()).into_result() {
+                Ok(ast) => ast,
+                Err(error) => {
+                    return Err(Error::VerificationAstError(error));
+                }
+            };
 
         let mut ast_verifier = verify_ast::AstVerifier::new();
         if !ast_verifier.compare(input_ast, reparsed_output) {
@@ -383,7 +455,7 @@ pub fn format_code(
     range: Option<Range>,
     verify_output: OutputVerification,
 ) -> Result<String, Error> {
-    let input_ast = match full_moon::parse(code) {
+    let input_ast = match full_moon::parse_fallible(code, config.syntax.into()).into_result() {
         Ok(ast) => ast,
         Err(error) => {
             return Err(Error::ParseError(error));
@@ -391,7 +463,7 @@ pub fn format_code(
     };
 
     let ast = format_ast(input_ast, config, range, verify_output)?;
-    let output = full_moon::print(&ast);
+    let output = ast.to_string();
 
     Ok(output)
 }
@@ -445,40 +517,4 @@ mod tests {
         .unwrap();
         assert_eq!(output, "local x = 1\n");
     }
-
-    #[test]
-    fn test_config_column_width() {
-        let new_config = Config::new().with_column_width(80);
-        assert_eq!(new_config.column_width(), 80);
-    }
-
-    #[test]
-    fn test_config_line_endings() {
-        let new_config = Config::new().with_line_endings(LineEndings::Windows);
-        assert_eq!(new_config.line_endings(), LineEndings::Windows);
-    }
-
-    #[test]
-    fn test_config_indent_type() {
-        let new_config = Config::new().with_indent_type(IndentType::Spaces);
-        assert_eq!(new_config.indent_type(), IndentType::Spaces);
-    }
-
-    #[test]
-    fn test_config_indent_width() {
-        let new_config = Config::new().with_indent_width(2);
-        assert_eq!(new_config.indent_width(), 2);
-    }
-
-    #[test]
-    fn test_config_quote_style() {
-        let new_config = Config::new().with_quote_style(QuoteStyle::ForceDouble);
-        assert_eq!(new_config.quote_style(), QuoteStyle::ForceDouble);
-    }
-
-    #[test]
-    fn test_config_call_parentheses() {
-        let new_config = Config::new().with_call_parentheses(CallParenType::None);
-        assert_eq!(new_config.call_parentheses(), CallParenType::None);
-    }
 }
