@@ -38,3 +38,14 @@ foo(
 -- https://github.com/JohnnyMorganz/StyLua/issues/611
 local function foo(): (number)
 end
+
+-- https://github.com/JohnnyMorganz/StyLua/issues/679
+type A = B & (C | D)
+type A = B & (C?)
+type A = ((string) -> string) & ((number) -> number)
+type A = (A | B)?
+type A = (A | B) -- comment
+
+-- https://github.com/JohnnyMorganz/StyLua/issues/729
+type SomeType<T..., U...> = (T...) -> U...
+local fn: SomeType<(string, number), (boolean)>
